      PROGRAM DRAGON
C     *******
C+---------------------------------------------------------------------+
C|                                                                     |
C|               PPPPPPP   DDDDDD    PPPPPPP   LL                      |
C|               PPPPPPPP  DDDDDDD   PPPPPPPP  LL                      |
C|               PP    PP  DD   DDD  PP    PP  LL                      |
C|               PP    PP  DD    DD  PP    PP  LL                      |
C|               PPPPPPPP  DD    DD  PPPPPPPP  LL                      |
C|               PPPPPPP   DD    DD  PPPPPPP   LL                      |
C|               PP        DD    DD  PP        LL                      |
C|               PP        DD   DDD  PP        LL                      |
C|               PP        DDDDDDD   PP        LLLLLLLL                |
C|               PP        DDDDDD    PP        LLLLLLLL                |
C|                                                                     |
C|               Copyright A.D. Murray & A.N. Fitch 1988               |
C|                                                                     |
C|                                                                     |
C|          PROGRAM DRAGON  version 1.0.0     dated  20-01-88          |
C|                  (DRAGON version 3.1 March 1987)                    |
C|                                                                     |
C|          Author(s): J. K. Cockcroft, ILL Grenoble France            |
C|                                                                     |
C|                   Copyright J.K. Cockcroft 1988                     |
C|                                                                     |
C|                             Disclaimer                              |
C|                                                                     |
C| Although this program  has  been  thoroughly  tested,  no  warranty |
C| (expressed  or  implied) is given concerning the proper functioning |
C| of the program or the reliability of the results produced.          |
C|                                                                     |
C|                              History                                |
C|                                                                     |
C| Version 1.0.0    (20-01-88) - First release for PDPL                |
C|                                                                     |
C+---------------------------------------------------------------------+
C
C  This version of DRAGON has been modified from the VAX version.
C  These modifications were necessary in order to have full FORTRAN 77
C  ANSI standard compatibility and to comply with PDPL standards.
C
C  The overall modifications are as follows:
C
C  1) A second mode of operation, namely energy-dispersive X-ray
C  scattering has been added. To enable this a new subroutine GETEDS
C  has been included and a variable EMAX has been included in COMMON 
C  block /HKL7/.
C
C  2) PARAMETER statements were originally of the form;
C     PARAMETER NDIM2=75
C  Parenthesis has been added thus;
C     PARAMETER (NDIM2=75)
C
C  3) Variable NTX has been removed from COMMON block /HKL9/ because
C  mixing of CHARACTERs and non-CHARACTERs is not allowed.
C
C  4) The use of a $ symbol to supress carriage-return and linefeed
C  in FORMAT statements has been removed. PDPL subroutine PROMPT is
C  employed instead.
C
C  5) Several changes have been made to the input and output of variables
C  so that standard PDPL routines such as COMINT and DIGITS can be used.
C  In particular subroutine GETVAL has been abandoned in favour of
C  subroutine DIGITS, which does much the same thing.
C
C  6) The program has been "softened" so that mistakes in input (eg. Cell
C  parameters, spacegroup etc.) no longer cause the program to abort and
C  another attempt at input is allowed.
C
C  8) Default cell parameters wavelength and zerpoint are read in from
C  current default file using PDPL routine DEFCEL.
C
C  9) All modifications are marked in the code by a comment card thus;
C  CMOD*
C  Most original code has been unchanged and commented out thus;
C  C#
C
C  These modifications should not affect the accurate operation of the
C  program.
C
C  Modifications made by A.D. Murray - Computer Centre U.C.L.
C  Gower Street London WC1             (04-01-88 to 11-01-88)
C
C
C                                  ------
C                                  DRAGON
C                                  ------
C
C
C                               Introduction
C                               ------------
C
C     Dragon is a simple program to generate the h,k,l reflection lists
C     in  the  refinement  of powder diffraction data.  The program requ
C     the unit cell parameters, wavelength of the radiation  used,  and
C     spacegroup symbol.  All the information needed to determine extinc
C     conditions and the multiplicities of  the  reflections  is  determ
C     solely  from  the spacegroup symbol.  The program is fully interac
C     and allows great flexabilty in the choice and  settings  of  the
C     cell.  Several choices of output are available.
C
C                           Unit cell parameters
C                           --------------------
C
C     The program can generate h,k,l reflection lists for any unit cell,
C     for triclinic, monoclinic (any axis unique), orthorhombic, tetrago
C     trigonal (including the use of either rhombohedral or  hexagonal
C     cells  for  rhombohedral  spacegroups),  hexagonal, and cubic syst
C     For tetragonal and hexagonal unit cells, the z-axis must be unique
C
C                  Interpretation of the spacegroup symbol
C                  ---------------------------------------
C
C     The first character of the spacegroup symbol is composed of  a  le
C     denoting the lattice type which must be one of the following :
C                               P A B C F I R
C     indicating primitive, A-face centred, B-face centred, C-face  cent
C     all   face   centred,   body   centred,   and   rhombohedral  latt
C     respectively.  The remaining characters  of  the  symbol  specify
C     point group and are composed of upto 3 symbols specifying the symm
C     relative to particular directions within the  spacegroup.   The  p
C     group  symbols must consist of one (or a combination) of the follo
C     symbols :
C                1  2  2(1) 3 -3  3(n) 4 -4  4(n) 6 -6  6(n)
C     indicating the presence of  rotation,  rotation-inversion,  and  s
C     axes, and
C                                m a b c n d
C     indicating mirror and glide planes.  A symbol composed of an axis
C     a plane is written with a slash separating the axis from the plane
C     2(1)/m.  Spaces are not permitted within the spacegroup symbol.
C
C                               Lattice type
C                               ------------
C
C     The  lattice  type  is  used  in  the  determination  of  the  gen
C     extinction conditions.  These are as follows :
C                    P => h,k,l : no conditions
C                    A => h,k,l : k + l = 2n
C                    B => h,k,l : h + l = 2n
C                    C => h,k,l : h + k = 2n
C                    F => h,k,l : h + k = 2n, k + l = 2n
C                    I => h,k,l : h + k + l = 2n
C                    R => h,k,l : no conditions (rhombohedral cell)
C                                 - h + k + l = 3n (hexagonal cell)
C     Restraints are placed on the lattice type for certain crystal syst
C
C                       Determination of crystal type
C                       -----------------------------
C
C     This is determined from the 3 symbols making up the remainder  of
C     spacegroup  symbol.   In  order to determine the crystal type, cer
C     conventions have to be adhered to :
C
C     Triclinic : only a single monad axis is permitted.
C
C     Monoclinic : the unique axis must have a  diad  axis  and/or
C          plane.   Short  or full symbols may be used eg P2(1) or
C          P12(1)1.  Where the short symbol is  used,  the  unique
C          axis  is  determined from the unit cell parameters.  In
C          the full symbol, the symbols refer to the x, y,  and  z
C          axes respectively.
C
C     Orthorhombic : each symbol is composed of a diad axis and/or
C          plane  of  symmetry  with  reference to the x, y, and z
C          axes respectively.
C
C     Tetragonal : primary symbol must be at least a tetrad  axis.
C          The  remaining  symbols  may  either  be blank eg P4 or
C          consist of diad axes and/or planes  eg  P422.   Symbols
C          refer   to   [001],   [100],   and   [110]   directions
C          respectively.
C
C     Trigonal : primary symbol must be at  least  a  triad  axis.
C          The  remaining  symbols  may  be  either blank eg P3 or
C          consist of a diad axis or mirror  plane  with  a  monad
C          axis  eg P312 and P3m1.  Symbols refer to [001], [...],
C          and [110] directions respectively.
C
C     Hexagonal : primary symbol must be at least  a  hexad  axis.
C          The  remaining symbols may be blank eg P6(3) or consist
C          of diad axes and/or planes.  Symbols  refer  to  [001],
C          [...], and [...] directions respectively.
C
C     Cubic : primary symbol may be either  diad  or  tetrad  axis
C          and/or  plane  and  refers  to  the  [100]  and related
C          directions.  The secondary symbol must be a  triad  and
C          refers  to  the  [111]  direction.  The tertiary symbol
C          must be a diad axis and/or plane if the primary  symbol
C          is  at  least a tetrad axis and refers to the [110] and
C          related directions.
C
C     The crystal type deduced from  the  pointgroup  is  then  checked
C     against the cell parameters.
C
C          Special extinctions derived from the pointgroup symbols
C          -------------------------------------------------------
C
C     Only symmetry operations which generate translational  components
C     for  the  symmetry operator will cause extinction to occur.  Thus
C     the following symbols are ignored when searching for extinction :
C                     1 -1  2  m  3 -3  4 -4  6 and -6.
C     The remaining symmetry operators will always  result  in  one  or
C     more  extinction conditions as shown below.  Note that the actual
C     symmetry operators will vary depending upon  the  the  choice  of
C     origin.   Extinction  conditions  shown in brackets have symmetry
C     equivalents and are thus redundant in this program.
C
C     Monoclinic.
C
C         Primary symbol or symbol refering to the x-axis.
C            2(1) =>  1/2+x,    -y,    -z.  h00 : h = 2n.
C            b    =>     -x, 1/2+y,     z.  0kl : k = 2n.
C            c    =>     -x,     y, 1/2+z.  0kl : l = 2n.
C            n    =>     -x, 1/2+y, 1/2+z.  0kl : k + l = 2n.
C         Secondary symbol or symbol refering to the y-axis.
C            2(1) =>     -x, 1/2+y,    -z.  0k0 : k = 2n.
C            a    =>  1/2+x,    -y,     z.  h0l : h = 2n.
C            c    =>      x,    -y, 1/2+z.  h0l : l = 2n.
C            n    =>  1/2+x,    -y, 1/2+z.  h0l : h + l = 2n.
C         Tertiary symbol or symbol refering to the z-axis.
C            2(1) =>     -x,    -y, 1/2+z.  00l : l = 2n.
C            a    =>  1/2+x,     y,    -z.  hk0 : h = 2n.
C            b    =>      x, 1/2+y,    -z.  hk0 : k = 2n.
C            n    =>  1/2+x, 1/2+y,    -z.  hk0 : h + k = 2n.
C
C     Orthorhombic.
C
C         Primary symbol.
C            2(1) =>  1/2+x,    -y,    -z.  h00 : h = 2n.
C            b    =>     -x, 1/2+y,     z.  0kl : k = 2n.
C            c    =>     -x,     y, 1/2+z.  0kl : l = 2n.
C            n    =>     -x, 1/2+y, 1/2+z.  0kl : k + l = 2n.
C            d    =>     -x, 1/4+y, 1/4+z.  0kl : k + l = 4n.
C
C         Secondary symbol.
C            2(1) =>     -x, 1/2+y,    -z.  0k0 : k = 2n.
C            a    =>  1/2+x,    -y,     z.  h0l : h = 2n.
C            c    =>      x,    -y, 1/2+z.  h0l : l = 2n.
C            n    =>  1/2+x,    -y, 1/2+z.  h0l : h + l = 2n.
C            d    =>  1/4+x,    -y, 1/4+z.  h0l : h + l = 4n.
C
C         Tertiary symbol.
C            2(1) =>     -x,    -y, 1/2+z.  00l : l = 2n.
C            a    =>  1/2+x,     y,    -z.  hk0 : h = 2n.
C            b    =>      x, 1/2+y,    -z.  hk0 : k = 2n.
C            n    =>  1/2+x, 1/2+y,    -z.  hk0 : h + k = 2n.
C            d    =>  1/4+x, 1/4+y,    -z.  hk0 : h + k = 4n.
C
C     Tetragonal.
C
C         Primary symbol.
C            4(1) =>     -y,     x, 1/4+z.  00l : l = 4n.
C            4(2) =>     -y,     x, 1/2+z.  00l : l = 2n.
C            4(3) =>     -y,     x, 3/4+z.  00l : l = 4n.
C            a    =>  1/2+x,     y,    -z.  hk0 : h = 2n, k = 2n.
C            b = a
C            n    =>  1/2+x, 1/2+y,    -z.  hk0 : h + k = 2n.
C            d    =>  1/4+x, 1/4+y,    -z.  hk0 : h + k = 4n.
C
C         Secondary symbol.
C            2(1) =>  1/2+x,    -y,    -z.  h00 : h = 2n.
C                                          (0k0 : k = 2n.)
C            a    =>  1/2+x,    -y,     z.  h0l : h = 2n.
C                                          (0kl : k = 2n.)
C            b = a
C            c    =>      x,    -y, 1/2+z.  h0l : l = 2n.
C                                          (0kl : l = 2n.)
C            n    =>  1/2+x,    -y, 1/2+z.  h0l : h + l = 2n.
C                                          (0kl : k + l = 2n.)
C            d    =>  1/4+x,    -y, 1/4+z.  h0l : h + l = 4n.
C                                          (0kl : k + l = 4n.)
C
C         Tertiary symbol.
C            2(1) =>  1/4+y, 1/4+x,    -z.  hh0 : h = 2n.
C            a    =>  1/4+y, 1/4+x,     z.  hhl : h = 2n.
C            b = a
C            c    =>      y,     x, 1/2+z.  hhl : l = 2n.
C            n    =>  1/4+y, 1/4+x, 1/2+z.  hhl : h + l = 2n.
C            d    =>  1/8+y, 1/8+x, 1/2+z.  hhl : 2h + l = 2n.
C
C     Trigonal (rhombohedral)
C
C         Secondary symbol.
C            c    =>  1/2+y, 1/2+x, 1/2+z.  hhl : l = 2n.
C                                           hkk : h = 2n.
C
C     Trigonal (hexagonal)
C
C         Primary symbol.
C            3(1) =>     -y,   x-y, 1/3+z.  00l : l = 3n.
C            3(2) =>     -y,   x-y, 2/3+z.  00l : l = 3n.
C
C         Secondary symbol.
C            c    =>     -y,    -x, 1/2+z.  h0l : l = 2n.
C                                           0kl : l = 2n.
C
C         Tertiary symbol.
C            c    =>      y,     x, 1/2+z.  hhl : l = 2n.
C                                         2h-hl : l = 2n.
C
C     Hexagonal
C
C         Primary symbol.
C            6(1) =>    x-y,     x, 1/6+z.  00l : l = 6n.
C            6(2) =>    x-y,     x, 1/3+z.  00l : l = 3n.
C            6(3) =>    x-y,     x, 1/2+z.  00l : l = 2n.
C            6(4) =>    x-y,     x, 2/3+z.  00l : l = 3n.
C            6(5) =>    x-y,     x, 5/6+z.  00l : l = 6n.
C
C         Secondary symbol.
C            c    =>     -y,    -x, 1/2+z.  h0l : l = 2n.
C
C         Tertiary symbol.
C            c    =>      y,     x, 1/2+z.  hhl : l = 2n.
C
C     Cubic
C
C         Primary symbol.
C            2(1) =>  1/2+x,    -y,    -z.  h00 : h = 2n.
C                                          (0k0 : k = 2n.)
C                                          (00l : l = 2n.)
C            4(1) =>       ,      ,      .  h00 : h = 4n.
C                                          (0k0 : k = 4n.)
C                                          (00l : l = 4n.)
C            4(2) =>       ,      ,      .  h00 : h = 2n.
C                                          (0k0 : k = 2n.)
C                                          (00l : l = 2n.)
C            4(3) =>       ,      ,      .  h00 : h = 4n.
C                                          (0k0 : k = 4n.)
C                                          (00l : l = 4n.)
C            a    =>       ,      ,      .  hk0 : h = 2n.
C                                           h0l : l = 2n.
C                                          (0kl : k = 2n.)
C            b = a
C            c = a
C            n    =>       ,      ,      .  hk0 : h + k = 2n.
C                                           h0l : h + l = 2n.
C                                          (0kl : k + l = 2n.)
C            d    =>       ,      ,      .  hk0 : h + k = 4n.
C                                           h0l : h + l = 4n.
C                                          (0kl : k + l = 4n.)
C
C         Tertiary symbol.
C            a    =>       ,      ,      .  hhl : l = 2n.
C                                           hkk : h = 2n.
C                                          (hkh : k = 2n.)
C            b = a
C            c = a
C            n = a
C            d    =>       ,      ,      .  hhl : 2h + l = 4n.
C                                           hkk : h + 2k = 4n.
C                                           (kh : 2h + k = 4n.)
C
C                                Wavelength
C                                ----------
C
C     This determines the maximum values of h, k, and  l  used  in  the
C     generation of the reflection list.
C
C           Reflections generated for the different crystal types
C           -----------------------------------------------------
C
C     The following conditions are placed on the  generation  of  h,k,l
C     values for the different crystal types :
C
C           1 Triclinic      l > 0
C                         or k >= l = 0
C                         or h >= k = l = 0
C           2 Monoclinic     h >= 0 and k >= 0 and l >= 0
C                         or h >= 0 and k > 0 and l < 0
C           3                h >= 0 and k >= 0 and l >=0
C                         or k >= 0 and l > 0 and h < 0
C           4                h >= 0 and k >= 0 and l >=0
C                         or l >= 0 and h > 0 and k < 0
C           5 Orthorhombic   h >=0 and k >= 0 and l >= 0
C           6 Tetragonal     h > 0 and k = 0 and l >= 0
C                         or h >= k > 0 and l >= 0
C           7                h >= k >= 0 and l >= 0
C           8 Trigonal
C           9
C          10
C          11
C          12
C          13 Hexagonal      h >= 0 and k >= 0 and l >= 0
C          14                h >= k > = 0 and l >= 0
C          15 Cubic          h >= k >= 0 and h >= l >= 0
C          16                h >= k >= l >= 0
C
C         Multiplicities generated for the different crystal types
C         --------------------------------------------------------
C
C     The multiplicities assigned to a reflection are chosen  according
C     to the following table :
C
C           1 Triclinic    hkl =  2
C           2 Monoclinic   h00 =  2;  0kl =  2;  hkl =  4
C           3              0k0 =  2;  h0l =  2;  hkl =  4
C           4              00l =  2;  hk0 =  2;  hkl =  4
C           5 Orthorhombic h00 =  2;  0k0 =  2;  00l =  2;  hk0 =  4;
C                          h0l =  4;  0kl =  4;  hkl =  8
C           6 Tetragonal   00l =  2;  h00 =  4;  0k0 =  4;  hk0 =  4;
C                          hkl =  8
C           7              00l =  2;  h00 =  4;  hh0 =  4;  h0l =  8;
C                          hk0 =  8;  hhl =  8;  hkl = 16
C           8 Trigonal     hhh =  2;  hkl =  6
C           9              hhh =  2;  hhl =  6;  hkk =  6; -h0h =  6;
C                          hkl = 12
C          10              00l =  2;  hkl =  6
C          11              00l =  2;  h0l =  6;  0kl =  6;  hh0 =  6;
C                          hkl = 12
C          12              00l =  2;  h00 =  6;  hhl =  6;2h-hl =  6;
C                          hkl = 12
C          13 Hexagonal    00l =  2;  h00 =  6;  hh0 =  6;  hk0 =  6;
C                          hkl = 12
C          14              00l =  2;  h00 =  6;  hh0 =  6;  hk0 = 12;
C                          h0l = 12;  hhl = 12;  hkl = 24
C          15 Cubic        h00 =  6;  hhh =  8;  hh0 = 12;  hk0 = 12
C                          hkl = 24
C          16              h00 =  6;  hhh =  8;  hh0 = 12;  hk0 = 24;
C                          hhl = 24;  hkk = 24;  hkl = 48
C
C                            Running the program
C                            -------------------
C
C     a) Input
C
C     The program wil request the following information :
C       1. A title of upto 75 characters.  If <CR> is given, a  default
C          title giving date and time will be used.
C       2. The cell constants a,  b,  c,  alpha,  beta,  &  gamma.   No
C          default  is available.  If any angle is unspecified or zero,
C          its value is set to 90 degrees.  If b is zero, its value  it
C          set  to a.  If b and c are unspecified or zero, both are set
C          equal to a.
C       3. The wavelength, maximum value of two-theta required, and the
C          zero-point  correction.   The  latter is applied as :  obs =
C          calc +  zero-point.   If  lambda  is  zero,  copper  K-alpha
C          radiation  is  assumed.   If two-theta maximum is zero, then
C          180 degrees is assumed.
C       4. The spacegroup in symbol form.   If  <CR>,  then  a  default
C          spacegroup  consistent  with  the lattice parameters will be
C          chosen.
C       5. Information relevant to output.
C
C     b) Output
C
C       1. The  reflections  may  be  sorted  in  order  of  decreasing
C          d-spacing.
C       2. Reflections may be written to lineprinter  output  or  to  a
C          file.   Two  formats  are  available  for  the  latter.  The
C          multipattern Rietveld program  may  require  more  than  one
C          reflection  listing,  so  the  program will loop back to the
C          start when more than one pattern is  requested.   Output  to
C          file  is  in  the  format  (5I8) for both options, where the
C          parameters are  codeword,  h,  k,  l,  &  multiplicity,  the
C          codeword   for   the   multi-pattern  containing  additional
C          information.
C
C                               Installation
C                               ------------
C
C     The program is written in simple Fortran and should  run  on  any
C     computor  with interactive facilities.  (It can be easily adapted
C     to run as a  batch  job.)  The  only  non-standard  features  are
C     Fortran  calls  to DATE and TIME which need to be supplied by the
C     user, but may be  omitted  if  non-available.  The   program   is
C     currently dimensioned  for  4000  reflections.   To  change this,
C     redimension  the arrays IH, IK, IL, MUL, D, TH, ICD, and IND, and
C     set MAXHKL to the new array size using the parameter NDIM1.
C
C                                 Guarantee
C                                 ---------
C
C     This program has been  extensively  tested  for  all  spacegroups
C     using  the  symbols  given  in the International Tables for X-ray
C     Crystallography for the standard spacegroup  settings.   However,
C     it  is the user's own responsibilty to check that the information
C     generated by this program is consistent with the spacegroup under
C     consideration.    The   program  may  be  altered,  modified,  or
C     subsequently changed from any form other than  that  supplied  by
C     the  author,  at  the users own risk !  However, it is considered
C     extremely  unlikely  that   minor   changes   relating   to   the
C     input/output  sections  of  the  source  code  will result in any
C     errors in the generation of hkl reflection lists.
C
C
C
C          An earlier version of this program, written in Algol  68  by
C     J.W.Talbot,  has  existed  at Oxford university for several years
C     now.  The Fortran code for this version is  completely  original,
C     the  code  from  the  above  version  not  being available to the
C     author, and so only reflects some of the  concepts  envisaged  in
C     the earlier work.
C
C        Jeremy Karl Cockcroft. Institut Laue Langevin. March 1987.
C
C     *******
C     Use standard variable types for reals and integers.
C     NB Characters are explicitly defined.
      IMPLICIT REAL*4 (A-H,O-Z)
      IMPLICIT INTEGER*4 (I-N)
C     Set dimension size.
      PARAMETER (NDIM1=4000)
      CHARACTER*1 ICH,ISPG,ISYM,IDEF,ANS
CMOD*
C     Changed to CHARACTER*30 for PDPL compatiblity (Declared later)
C     A.D. Murray (04-01-88)
C#    CHARACTER*40 AFILE
      COMMON/HKL1/ICH(24),ISPG(20),ISYM(6,3),IDEF(6)
      COMMON/HKL2/IH(NDIM1),IK(NDIM1),IL(NDIM1),MUL(NDIM1),D(NDIM1),
     +  TH(NDIM1),IND(NDIM1),ICD(NDIM1)
      COMMON/HKL3/IOR,IOW,LPT,IOUT
      COMMON/HKL4/PI,RADS,RADS2,NBIT,NREF,NTOT,NCYC,MAXHKL
CMOD*
C     Variables not required - PDPL routine GETDAT used instead
C     of VAX routines DATE and TIME (see later)
C     A.D. Murray (04-01-88)
C#    CHARACTER*9 DAY
C#    CHARACTER*8 TIM
CMOD*
C     Extra variables for PDPL routines
C     A.D. Murray (04-01-88)
      COMMON /PDPL1/ LPTFIL,AFILE,HLPFIL
      CHARACTER*30   LPTFIL,AFILE,HLPFIL
      COMMON /PDPL2/ DFCELL(6),DFWAV,DZP
      COMMON /PDPL3/ DFCAOK,ENERGY
      LOGICAL        DFCAOK,ENERGY
      CHARACTER*80   LINEIN,STRING,GETSTR
      CHARACTER*40   BUTBIT(2)
      CHARACTER*30   DEFFIL,CELFIL,BFILE(4)
      CHARACTER*20   DATIME
      DOUBLE PRECISION VAL(1)
      DIMENSION      IUNIT(6)
      LOGICAL        ERROR
CMOD*
C     End of extra variable list

C     Set array limits.
CMOD*
C     Note: The following statement was changed from;
C#    DATA MAXHKL /NDIM1/
C     because MAXHKL is in COMMON. A.D. Murray (04-01-88)
      MAXHKL = NDIM1
CMOD*
C     The following statement has been moved to a BLOCK DATA module
C     A.D. Murray (04-01-88)
C#    DATA ICH/'P','A','B','C','F','I','R','1','2','3','4','5','6',
C#   +  'm','a','b','c','n','d','-','/','(',')',' '/
C     Initialize.
CMOD*
C     Logs use of program - not used by A.D. Murray (04-01-88)
C#    CALL LOGUSE
C     Start the clock.
CMOD*
C     CPU usage monitor - not used by A.D. Murray (04-01-88)
C#    CALL INICPU
CMOD*
C     Files/units set by PDPL routine PDPLFS (See later)
C     A.D. Murray (04-01-88)
C     Read TTY.
C#    IOR = 5
C     Write TTY.
C#    IOW = 6
C     LPT listing.
C#    LPT = 16
C     Open file as simple text file.
CMOD*
C     Use PDPL routine GFILE for file OPEN/CLOSE
C     A.D. Murray (04-01-88)
C#    OPEN(UNIT=LPT,
C#   +     FILE='DRAGON.LIS',
C#   +     STATUS='NEW',
C#   +     CARRIAGE CONTROL='LIST')
C     Channel for reflection file list (eg. HKL.DAT).
C#    IOUT = 21

CMOD*
C     Modified section
C     Set up input/output units and files using PDPL routine PDPLFS
C     A.D. Murray (04-01-88)

      CALL PDPLFS(5,4,IUNIT,BFILE)

C     Terminal input/output

      IOR = IUNIT(1)
      IOW = IUNIT(2)

C     Line printer listing unit and file name

      LPT = IUNIT(3)
      LPTFIL = BFILE(1)

C     Output file unit and file name

      IOUT = IUNIT(4)
      DEFFIL = BFILE(2)

C     Help file unit and file name

      IHELP = IUNIT(5)
      HLPFIL = BFILE(3)

C     Default cell parameters file

      CELFIL = BFILE(4)

C     Open lineprinter file.

      CALL GFILE(LPT,LPTFIL,'CNV','FOR',0,ERROR,IOW)

C     Error trap for file OPENing
C     A.D. Murray (04-01-88)

          IF (ERROR) THEN
          STOP
          ENDIF

C     Set up PDPL files/units

      CALL SETALL(IOR,IOW,IHELP,HLPFIL)

CMOD*
C     End of modification section

C     Set total number of reflections to zero.
      NTOT = 0
C     Useful conversion constants.
      PI = ATAN(1.0) * 4.0
      RADS = PI / 180.0
      RADS2 = RADS / 2.0
C     Version 1.0 April 1983.
C     Version 2.0 June 1984.
C     Version 3.0 September 1986.
      WRITE(IOW,1)
CMOD*
C     PDPL type banner
C     A.D. Murray (07-01-88)
C#  1 FORMAT(/1X,'DRAGON version 3.1 March 1987 (J.K.C.)'/
C#   +  /1X,'For details on use, ',
C#   +  'type DUA4:[C5.COCKCROFT.LIB]DRAGON.HLP')
    1 FORMAT(/22X,'DRAGON - Reflection Generation Program'/
     +        22X,'======================================'/
     +       /22X,'   PDPL version 1.0.0    (20-01-88)'/
     +        21X,'(DRAGON version 3.1 March 1987 (J.K.C.))'/
     +       /32X,'(Type ? for HELP)')
      WRITE(LPT,2)
    2 FORMAT(/10X,'***** Program DRAGON version 3.1 March 1987 *****',
     +       /10X,'             (Jeremy Karl Cockcroft)             ')
CMOD*
C     Default unit cell etc.
C     A.D. Murray (08-01-88)
      CALL DEFCEL(CELFIL,DFCELL,DFWAV,DZP,DFCAOK,IOW)
CMOD*
C     Option for energy dispersive mode - Extra section
C     A.D. Murray (11-01-88)
  350 CONTINUE
      WRITE(IOW,4000)
 4000 FORMAT(/' Enter calculation mode'/
     &        '   M = Monochromatic (Normal diffraction mode)'/
     &        '   E = Energy dispersive X-ray diffraction'/
     &        ' (<CR> = Monochromatic)')
      CALL PROMPT('Mode > ',7,IOW)
      CALL COMINT(0,LINEIN,IFLAG,'MODE  ')
      IF (IFLAG.EQ.2) CALL EOFINP
      IF (IFLAG.EQ.3) GOTO 350
      ANS = LINEIN(1:1)

          IF (ANS.EQ.'E') THEN
          ENERGY = .TRUE.
          ELSE
          ENERGY = .FALSE.
          ENDIF
CMOD*
C     End of modified section
CMOD*
C     Extra statement label
C     A.D. Murray (06-01-88)
 300  CONTINUE
      WRITE(IOW,3)
CMOD*
C     Use PDPL routine PROMPT instead of VAX $ 
C     A.D. Murray (07-01-88)
C#  3 FORMAT(/1X,'Is generation of Rietveld input-file required ?',
C#   +  1X,'(<CR> = No)'/1X,'->  ',$)
    3 FORMAT(/1X,'Is generation of Rietveld input-file required ?',
     +  1X,'(<CR> = No)')
      CALL PROMPT('Output ? > ',11,IOW)
CMOD*
C     Use PDPL routine COMINT to get input
C     A.D. Murray (06-01-88)
C#    READ(IOR,4) ANS
    4 FORMAT(A1)
      CALL COMINT(0,LINEIN,IFLAG,'OUTPUT')
      IF (IFLAG.EQ.2) CALL EOFINP
      IF (IFLAG.EQ.3) GOTO 300
      ANS = LINEIN(1:1)
CMOD*
C     End of modified section
C     Simply set i/o channel number to zero for no output.
      IF (ANS.NE.'Y'.AND.ANS.NE.'y') IOUT = 0
C     If yes, then get filename.
      IF (IOUT.EQ.0) GOTO 8
C     Get filename and format.
CMOD*
C     Extra statement label
C     A.D. Murray (06-01-88)
 301  CONTINUE
CMOD*
C     Changed to write out variable DEFFIL
C     Use PDPL routine PROMPT instead of VAX $ 
C     A.D. Murray (07-01-88)
C#    WRITE(IOW,5)
C#  5 FORMAT(/1X,'Give output filename (<CR> = HKL.DAT)'/1X'->  ',$)
      BUTBIT(1) = 'Give output filename (<CR> = '
      BUTBIT(2) = ')'
      STRING = GETSTR(BUTBIT(1),DEFFIL,BUTBIT(2),30)
      WRITE(IOW,5) STRING
    5 FORMAT(/1X,A80)
      CALL PROMPT('File > ',7,IOW)
CMOD*
C     Use PDPL routine COMINT to get input
C     A.D. Murray (06-01-88)
C#    READ(IOR,6) AFILE
    6 FORMAT(A30)
      CALL COMINT(0,LINEIN,IFLAG,'FILE  ')
      IF (IFLAG.EQ.2) CALL ABORT
      IF (IFLAG.EQ.3) GOTO 301
      AFILE = LINEIN(1:30)
CMOD*
C     End of modified section
C     Take default if required.
      IF (AFILE.EQ.'                                        ')
     +  AFILE = DEFFIL
CMOD*
C     Use PDPL routine GFILE for file OPEN/CLOSE
C     A.D. Murray (04-01-88)
C     Open file as simple text file.
C#    OPEN(UNIT=IOUT,
C#   +     FILE=AFILE,
C#   +     STATUS='NEW',
C#   +     CARRIAGE CONTROL='LIST')
      CALL GFILE(IOUT,AFILE,'CNV','FOR',0,ERROR,IOW)

CMOD*
C     Error trap for file OPENing
C     A.D. Murray (04-01-88)

      IF (ERROR) GOTO 301

C     Output file type ?
CMOD*
C     Extra statement label
C     A.D. Murray (06-01-88)
 302  CONTINUE
      WRITE(IOW,7)
    7 FORMAT(/1X,'Give output file format'
     +  /3X,'1 = Hewat     (PROF1  format)'
     +  /3X,'2 = Bendall   (MPREP  format)'
     +  /3X,'3 = Cockcroft (PREPRO format)'
CMOD*
C     Added 4th format
C     A.D. Murray (07-01-88)
     +  /3X,'4 = Miller indices (H,K,L) and 2Theta (or energy) values'
CMOD*
C     Default changed.
C     Use PDPL routine PROMPT instead of VAX $ 
C     A.D. Murray (07-01-88)
C#   +  /1X,'(<CR> = 3)'/1X,'->  ',$)
     +  /1X,'(<CR> = 2)')
      CALL PROMPT('Format > ',9,IOW)
CMOD*
C     Use PDPL routines COMINT and DIGITS to get input
C     A.D. Murray (06-01-88)
      CALL COMINT(0,LINEIN,IFLAG,'FORMAT')
      IF (IFLAG.EQ.2) CALL EOFINP
      IF (IFLAG.EQ.3) GOTO 302
      CALL DIGITS(LINEIN,VAL,1,1,KEND,NFND,ERROR,IOW)
      IF (ERROR) GOTO 302
      IFORM = VAL(1)
      IF (IFORM.LE.0.OR.IFORM.GT.4) IFORM = 2
CMOD*
C     End of modified section
C#    READ(IOR,4,END=33) ANS
C     Take default if required.
C#    IFORM = 3
C#    IF (ANS.EQ.'1') IFORM = 1
C#    IF (ANS.EQ.'2') IFORM = 2
C     Machine dependant subroutines.
CMOD*
C     PDPL routine GETDAT used instead of VAX routines DATE and TIME
C     A.D. Murray (04-01-88)
C#  8 CALL DATE(DAY)
C#    CALL TIME(TIM)
C#    WRITE(LPT,9) DAY,TIM
C   9 FORMAT(/'Output generated on ',A9,' at ',A8)
    8 CALL GETDAT(DATIME)
      WRITE(LPT,9) DATIME
    9 FORMAT(/'Output generated on ',A20)
C     How many reflection data sets ?
CMOD*
C     Extra statement label
C     A.D. Murray (06-01-88)
 303  CONTINUE
      WRITE(IOW,10)
CMOD*
C     Use PDPL routine PROMPT instead of VAX $ 
C     A.D. Murray (07-01-88)
C# 10 FORMAT(/1X,'Give number of reflection data sets',
C#   +  1X,'(<CR> = 1)'/1X,'->  ',$)
   10 FORMAT(/1X,'Give number of reflection data sets',
     +  1X,'(<CR> = 1)')
      CALL PROMPT('Number > ',9,IOW)
CMOD*
C     Use PDPL routines COMINT and DIGITS to get input
C     A.D. Murray (06-01-88)
C#    READ(IOR,4,END=33) ANS
C     Convert to integer.
C#    NCYC = ICHAR(ANS) - 48
      CALL COMINT(0,LINEIN,IFLAG,'NUMBER')
      IF (IFLAG.EQ.2) GOTO 33
      IF (IFLAG.EQ.3) GOTO 303
      CALL DIGITS(LINEIN,VAL,1,1,KEND,NFND,ERROR,IOW)
      IF (ERROR) GOTO 303
      NCYC = VAL(1)
CMOD*
C     End of modified section
C     Set maximum number to 3.
      IF (NCYC.LT.1.OR.NCYC.GT.3) NCYC = 1
      WRITE(LPT,11) NCYC
   11 FORMAT(/'Number of reflection data sets generated ',I1)
C     Do for each data set.
      DO 18 ICYC=1,NCYC
      WRITE(IOW,12) ICYC
   12 FORMAT(/1X,'Generating data set number ',I1)
C     Get title for LPT output.
      CALL TITLES
C     Get unit-cell parameters.
      CALL CELLIN
C     Choose a default spacegroup taken from lattice parameters.
      CALL DEFSPG
CMOD*
C     Allow for energy dispersive calculation with IF block.
C     A.D. Murray (11-01-88)
      IF (ENERGY) THEN
C     Get detector angle & energy range 
      CALL GETEDS
C     Get wavelength, 2-theta range, etc.
      ELSE
      CALL GETWAV
      ENDIF
C     Get spacegroup.
CMOD*
C     Extra statement label - A.D. Murray (07-01-88)
 400  CONTINUE
      CALL GETSPG
C     Determine Laue type.
      CALL GETLAU(ERROR)
CMOD*
C     Allow another go if error in GETLAU
C     A.D. Murray (07-01-88)
      IF (ERROR) GOTO 400
C     Write out Laue group.
      CALL LAUEGP
C     Check cell constants against spacegroup.
      CALL CHECKC
C     Get alternative cell constants for rhomobohedral systems.
      CALL RHOMBO
C     Determine lattice extinctions.
      CALL GENEXT
C     Determine symmetry extinctions.
      CALL SPEEXT
C     Write out extinction conditions.
      CALL EXTINC
C     Generate reflections.
      CALL HKLGEN
C     If no reflections generated, skip remaining subroutines.
      IF (NREF.EQ.0) GOTO 18
C     Write out reflection list.
      CALL LPTREF
C     Skip if no further output required.
      IF (IOUT.EQ.0.AND.NCYC.EQ.1) GOTO 18
CMOD*
C     Skip next bits if IFORM = 4
C     A.D. Murray (07-01-88)
      IF (IFORM.EQ.4) GOTO 16
CMOD*
C     Extra statement label
C     A.D. Murray (06-01-88)
 304  CONTINUE
C     Get reflection type for use in Rietveld.
      WRITE(IOW,13)
   13 FORMAT(/1X,'Give reflection type - code is :'
     +  /3X,'1 = Nuclear scattering only'
     +  /3X,'2 = Magnetic scattering only'
     +  /3X,'3 = Nuclear & magnetic scattering'
     +  /3X,'4 = X-ray alpha-1 scattering'
     +  /3X,'5 = X-ray alpha-2 scattering'
CMOD*
C     Use PDPL routine PROMPT instead of VAX $ 
C     A.D. Murray (07-01-88)
C#   +  /1X,'(<CR> = Nuclear scattering only)'/1X,'->  ',$)
     +  /1X,'(<CR> = Nuclear scattering only)')
      CALL PROMPT('Code > ',7,IOW)
CMOD*
C     Use PDPL routines COMINT and DIGITS to get input
C     A.D. Murray (06-01-88)
C#    READ(IOR,4,END=33) ANS
C#    NRT = ICHAR(ANS) - 48
      CALL COMINT(0,LINEIN,IFLAG,'CODE  ')
      IF (IFLAG.EQ.2) GOTO 33
      IF (IFLAG.EQ.3) GOTO 304
      CALL DIGITS(LINEIN,VAL,1,1,KEND,NFND,ERROR,IOW)
      IF (ERROR) GOTO 304
      NRT = VAL(1)
CMOD*
C     End of modified section
      IF (NRT.LT.1.OR.NRT.GT.5) NRT = 1
C     Pattern number does not apply if IFORM = 1.
      NPAT = 0
      IF (IFORM.EQ.1) GOTO 16
CMOD*
C     Extra statement label
C     A.D. Murray (06-01-88)
 305  CONTINUE
      WRITE(IOW,14)
CMOD*
C     Use PDPL routine PROMPT instead of VAX $ 
C     A.D. Murray (07-01-88)
C# 14 FORMAT(/1X,'Give pattern number (<CR> = 1)'/1X,'->  ',$)
   14 FORMAT(/1X,'Give pattern number (<CR> = 1)')
      CALL PROMPT('Pattern > ',10,IOW)
CMOD*
C     Use PDPL routines COMINT and DIGITS to get input
C     A.D. Murray (06-01-88)
C# 15 READ(IOR,4,END=33) ANS
C#    NPAT = ICHAR(ANS) - 48
      CALL COMINT(0,LINEIN,IFLAG,'PATTER')
      IF (IFLAG.EQ.2) GOTO 33
      IF (IFLAG.EQ.3) GOTO 305
      CALL DIGITS(LINEIN,VAL,1,1,KEND,NFND,ERROR,IOW)
      IF (ERROR) GOTO 305
      NPAT = VAL(1)
CMOD*
C     End of modified section
C     If <CR> take default.
      IF (NPAT.LT.1.OR.NPAT.GT.3) NPAT = 1
C     Calculate composite codeword.
   16 NCD = NPAT * 10 + NRT
C     And store it.
      DO 17 I=(NTOT+1),(NTOT+NREF)
   17 ICD(I) = NCD
C     Set NTOT to be the complete total.
      NTOT = NTOT + NREF
C     Next data set.
   18 CONTINUE
C     Skip if only 1 data set.
      IF (NCYC.EQ.1) GOTO 30
C     Resort if more than 1 data set.
      M = 1
      CALL SORTER(M,NTOT)
C     And write to LPT file.
      WRITE(IOW,19) NTOT
   19 FORMAT(/1X,'Total number of reflections of all data sets is ',I5)
      WRITE(LPT,20) NTOT
   20 FORMAT(/'Total number of reflections of all data sets is ',I5)
C     Write resorted list to LPT.
      WRITE(LPT,21)
   21 FORMAT(/9X,'Pattern 1',3X,'Pattern 2',3X,'Pattern 3'
     +  /2X,'Type',3(3X,'H',3X,'K',3X,'L'),4X,'J',
     +  8X,'D',5X,'2-theta'/)
      DO 28 I=1,NTOT
      NPAT = ICD(IND(I)) / 10
      NRT = ICD(IND(I)) - NPAT * 10
      GOTO (22,24,26) NPAT
   22 WRITE(LPT,23) NRT,IH(IND(I)),IK(IND(I)),IL(IND(I)),MUL(IND(I)),
     + D(IND(I)),TH(IND(I))
   23 FORMAT(I5,1X,3I4,24X,I5,1X,F10.4,1X,F8.2)
      GOTO 28
   24 WRITE(LPT,25) NRT,IH(IND(I)),IK(IND(I)),IL(IND(I)),MUL(IND(I)),
     + D(IND(I)),TH(IND(I))
   25 FORMAT(I5,1X,12X,3I4,12X,I5,1X,F10.4,1X,F8.2)
      GOTO 28
   26 WRITE(LPT,27) NRT,IH(IND(I)),IK(IND(I)),IL(IND(I)),MUL(IND(I)),
     + D(IND(I)),TH(IND(I))
   27 FORMAT(I5,1X,24X,3I4,I5,1X,F10.4,1X,F8.2)
   28 CONTINUE
C     Write out output files.
   30 IF (IOUT.EQ.0) GOTO 101
      CALL FILOUT(IFORM)
C     Close output file.
CMOD*
C     Use PDPL routine GFILE for file CLOSEing
C     A.D. Murray (04-01-88)
C#    CLOSE(UNIT=IOUT,DISPOSE='SAVE')
      CALL GFILE(IOUT,AFILE,'CLK','FOR',0,ERROR,IOW)
  101 WRITE(LPT,31)
   31 FORMAT(/'                              *******            ',
     +  '                  '
     +  /'Please notify the author if any errors are discovered',
     +  1X,'in the output'/)
C     Close output file.
CMOD*
C     Use PDPL routine GFILE for file CLOSEing
C     A.D. Murray (04-01-88)
C#    CLOSE(UNIT=LPT)
      CALL GFILE(LPT,LPTFIL,'CLK','FOR',0,ERROR,IOW)
CMOD*
C     Use variable file names LPTFIL and AFILE
C     A.D. Murray (06-01-88)
C#    WRITE(IOW,32)
C# 32 FORMAT(/1X,'Lineprinter listing is in file DRAGON.LIS')
      WRITE(IOW,32) LPTFIL
   32 FORMAT(/1X,'Lineprinter listing : ',A30)
      IF (IOUT.NE.0) WRITE(IOW,320) AFILE
  320 FORMAT( 1X,'Reflection file     : ',A30)
C     Normal end of program.
CMOD*
C     Use PDPL routine ENDALL for ending program
C     A.D. Murray (04-01-88)
C#    CALL ENDCPU
      CALL ENDALL('DRAGON    ')
C     EOF on input, abort to monitor !
   33 CALL EOFINP
C     *******
      END
      SUBROUTINE TITLES
C     *******
C     Use standard variable types for reals and integers.
C     NB Characters are explicitly defined.
      IMPLICIT REAL*4 (A-H,O-Z)
      IMPLICIT INTEGER*4 (I-N)
C     Input variables  : LPT    IOW    IOR
C     Output variables : TEXT   I
C     Subroutines used : EOFINP
C     Functions called :
      PARAMETER (NDIM2=75)
      CHARACTER*1 TEXT
      COMMON/HKL3/IOR,IOW,LPT,IOUT
      COMMON/HKL9/TEXT(NDIM2)
CMOD* Extra variables for PDPL routines
C     A.D. Murray (06-01-88)
      CHARACTER*80 LINEIN
CMOD*
C     The following statement has been removed and all references to
C     variable NTX have been changed to NDIM2 in the code.
C     (See modification notes at top of program)
C     A.D. Murray (04-01-88)
C#    DATA NTX/NDIM2/
C     Read in input data from TTY.
CMOD*
C     Extra statement label
C     A.D. Murray (06-01-88)
 306  CONTINUE
      WRITE(IOW,1)
CMOD*
C     Use PDPL routine PROMPT instead of VAX $ 
C     A.D. Murray (07-01-88)
C#  1 FORMAT(/1X,'Give title for output (<CR> = none)'
C#   +  /1X,'->  ',$)
    1 FORMAT(/1X,'Give title for output (<CR> = none)')
      CALL PROMPT('Title > ',8,IOW)
CMOD*
C     Use PDPL routine COMINT to get input
C     A.D. Murray (06-01-88)
C#    READ(IOR,2,END=6) TEXT
      CALL COMINT(1,LINEIN,IFLAG,'TITLE ')
      IF (IFLAG.EQ.2) GOTO 6
      IF (IFLAG.EQ.3) GOTO 306

        DO 307 IADM = 1,NDIM2
        TEXT(IADM) = LINEIN(IADM:IADM)
 307    CONTINUE

CMOD*
C     End of modified section
CMOD*
C     The following statement has been changed from;
C#  2 FORMAT(<NDIM2>A1)
C     because use of PARAMETER variables as edit descriptors in
C     FORMAT statements in not allowed in standard FORTRAN 77
C     A.D. Murray (04-01-88)
    2 FORMAT(75A1)
C     If blank, skip.
      DO 3 I=1,NDIM2
      IF (TEXT(I).NE.' ') GOTO 4
    3 CONTINUE
C     Return to calling subroutine.
      RETURN
C     Write out title to LPT.
    4 WRITE(LPT,5) TEXT
CMOD*
C     The following statement has been changed from;
C#  5 FORMAT(/<NDIM2>A1)
C     because use of PARAMETER variables as edit descriptors in
C     FORMAT statements in not allowed in standard FORTRAN 77
C     A.D. Murray (04-01-88)
    5 FORMAT(/75A1)
C     Return to calling subroutine.
      RETURN
C     EOF on input, abort to monitor !
    6 CALL EOFINP
C     *******
      END
      SUBROUTINE CELLIN
C     *******
C     Gets unit-cell parameters.
C     Use standard variable types for reals and integers.
C     NB Characters are explicitly defined.
      IMPLICIT REAL*4 (A-H,O-Z)
      IMPLICIT INTEGER*4 (I-N)
C     Input variables  : IOW    IOR    IOR    RADS   PI
C     Output variables : A      B      C      AL     BE     GA
C                        A1     B1     C1     S      SQ     V
C                        VR     SINA   SINB   SINC   AR     BR
C                        CR     COSA   COSB   COSC   COSAR  COSBR
C                        COSCR  AR2    BR2    CR2    BCR    CAR
C                        ABR    ALR    BER    GAR
C     Subroutines used : EOFINP
C     Functions called : SQRT   SIN    COS    ACOS
      PARAMETER (NDIM2=75)
      CHARACTER*1 TEXT
      COMMON/HKL3/IOR,IOW,LPT,IOUT
      COMMON/HKL4/PI,RADS,RADS2,NBIT,NREF,NTOT,NCYC,MAXHKL
      COMMON/HKL5/A,B,C,AL,BE,GA,COSA,AR2,BR2,CR2,BCR,CAR,ABR
      COMMON/HKL9/TEXT(NDIM2)
CMOD*
CMOD* Extra variables and statements for PDPL routines
C     A.D. Murray (02-01-88)
      COMMON /PDPL2/ DFCELL(6),DFWAV,DZP
      COMMON /PDPL3/ DFCAOK,ENERGY
      LOGICAL        DFCAOK,ENERGY
      CHARACTER*80 LINEIN
      DOUBLE PRECISION VAL
      LOGICAL ERROR
      DIMENSION VAL(6)
      DATA NVAL/6/
CMOD*
C     Extra statement label
C     A.D. Murray (06-01-88)
 308  CONTINUE
      WRITE(IOW,1)
    1 FORMAT(/1X,'Give cell constants (a b c alpha beta gamma)'
CMOD*
C     Use PDPL routine PROMPT instead of VAX $ 
C     A.D. Murray (07-01-88)
C#   +  /1X,'(Angles default to 90 if not specified)'/1X,'->  ',$)
     +  /1X,'(1 value entered => Cubic, 3 values entered =>',
     +   1X,'Orthorhombic)')
CMOD*
C     Default values from DEFCEL
C     A.D. Murray (08-01-88)

          IF (DFCAOK) THEN
          WRITE(IOW,3000) (DFCELL(I),I=1,6)
 3000     FORMAT(' (<CR> = ',3F9.4,1X,3F8.3,')')
          ENDIF

      CALL PROMPT('Cell > ',7,IOW)
CMOD*
C     Use PDPL routines COMINT and DIGITS to get input
C     A.D. Murray (06-01-88)
C#    READ(IOR,2,END=9) TEXT
CMOD*
C     The following statement has been changed from;
C#  2 FORMAT(<NDIM2>A1)
C     because use of PARAMETER variables as edit descriptors in
C     FORMAT statements in not allowed in standard FORTRAN 77
C     A.D. Murray (04-01-88)
    2 FORMAT(75A1)
C     Decode 6 reals.
C#    CALL GETVAL(NVAL,VAL)
      CALL COMINT(0,LINEIN,IFLAG,'CELL  ')
      IF (IFLAG.EQ.2) GOTO 9
      IF (IFLAG.EQ.3) GOTO 308

C     Apply default cell parameters (if available)

          IF (DFCAOK.AND.IFLAG.EQ.1) THEN
          DO 309 JADM = 1,6
          VAL(JADM) = DFCELL(JADM)
 309      CONTINUE
          GOTO 310
          ENDIF

      CALL DIGITS(LINEIN,VAL,6,1,KEND,NFND,ERROR,IOW)
      IF (ERROR) GOTO 308
 310  CONTINUE
CMOD*
C     End of modified section
      A = VAL(1)
      B = VAL(2)
      C = VAL(3)
      AL = VAL(4)
      BE = VAL(5)
      GA = VAL(6)
C     Negative lengths or angles are not allowed.
      IF (A.LT.0.0.OR.B.LT.0.0.OR.C.LT.0.0.OR.
     +    AL.LT.0.0.OR.BE.LT.0.0.OR.GA.LT.0.0) GOTO 7
C     Default lengths if zero to preceeding value.
      IF (B.EQ.0.0) B = A
      IF (C.EQ.0.0) C = B
C     Default angles if zero to 90.0.
      IF (AL.EQ.0.0) AL = 90.0
      IF (BE.EQ.0.0) BE = 90.0
      IF (GA.EQ.0.0) GA = 90.0
C     Determine reciprocal lattice parameters.
      A1 = AL * RADS
      B1 = BE * RADS
      C1 = GA * RADS
      S = (A1 + B1 + C1 ) / 2.0
C     Give error message if S > 180.
      IF (S.GE.PI) GOTO 7
      SQ = SQRT(SIN(S) * SIN(S - A1) * SIN(S - B1) * SIN(S - C1))
      V = 2.0 * A * B * C * SQ
C     Give error message if V is zero.
      IF (V.LE.0.0) GOTO 7
      WRITE(LPT,3) A,B,C,AL,BE,GA
    3 FORMAT(/'Cell constants (a b c alpha beta gamma) '
     +  /3F9.4,1X,3F8.3)
      WRITE(LPT,4) V
    4 FORMAT(/'Volume of unit cell is ',F7.1)
      VR = 1.0 / V
      SINA = SIN(A1)
      SINB = SIN(B1)
      SINC = SIN(C1)
      AR = B * C * SINA / V
      BR = C * A * SINB / V
      CR = A * B * SINC / V
      COSA = COS(A1)
      COSB = COS(B1)
      COSC = COS(C1)
      COSAR = (COSB * COSC - COSA) / (SINB * SINC)
      COSBR = (COSC * COSA - COSB) / (SINC * SINA)
      COSCR = (COSA * COSB - COSC) / (SINA * SINB)
      AR2 = AR * AR
      BR2 = BR * BR
      CR2 = CR * CR
      BCR = 2.0 * BR * CR * COSAR
      CAR = 2.0 * CR * AR * COSBR
      ABR = 2.0 * AR * BR * COSCR
      ALR = ACOS(COSAR) / RADS
      BER = ACOS(COSBR) / RADS
      GAR = ACOS(COSCR) / RADS
      WRITE(LPT,5) AR,BR,CR,ALR,BER,GAR
    5 FORMAT(/'Reciprocal lattice parameters (a* b* c* alpha* beta*',
     +  1X,'gamma*)'/3F8.5,3F7.2)
      WRITE(LPT,6) VR
    6 FORMAT(/'Volume of reciprocal unit-cell is ',F9.7)
C     Return to calling subroutine.
      RETURN
C     Give error message if error in cell constants.
    7 WRITE(IOW,8) A,B,C,AL,BE,GA
    8 FORMAT(/1X,'Impossible cell constants'
     +  /1X,3F9.4,1X,3F8.3)
CMOD*
C     Allow another attempt if mistakes are made !!
C     A.D. Murray (06-01-88)
C     Abort
C#    CALL ABORT
      GOTO 308
C     EOF on input, abort to monitor !
    9 CALL EOFINP
C     *******
      END
      SUBROUTINE GETVAL(NVAL,VAL)
C     *******
C     Returns the next real in the current string TEXT.
C     Use standard variable types for reals and integers.
C     NB Characters are explicitly defined.
      IMPLICIT REAL*4 (A-H,O-Z)
      IMPLICIT INTEGER*4 (I-N)
C     Input variables  : NVAL   TEXT   IOW
C     Output variables : VAL
C     Subroutines used :
C     Functions called :
      PARAMETER (NDIM2=75,NDIM3=10)
      CHARACTER*1 TEXT,DIGITS
      COMMON/HKL3/IOR,IOW,LPT,IOUT
      COMMON/HKL9/TEXT(NDIM2)
      DIMENSION VAL(NVAL)
      DIMENSION DIGITS(NDIM3)
      DATA DIGITS/'0','1','2','3','4','5','6','7','8','9'/
      DATA NDIG/NDIM3/
C     Set pointer in input string.
      N = 0
C     Do for each value required.
      DO 12 I=1,NVAL
C     Default to zero.
      VAL(I) = 0.0
C     If pointer is at end of string, then assume value is zero.
      IF (N.EQ.NDIM2) GOTO 12
C     Set the sign flag.
      NSGN = 0
C     Set the sign to positive.
      SGN = 1.0
C     Set the decimal point flag.
      NDP = 0
C     Spaces are the only separators.
    1 N = N + 1
C     Find the first non-space character.
      IF (TEXT(N).NE.' ') GOTO 2
C     If pointer is at end of string, then assume value is zero.
      IF (N.EQ.NDIM2) GOTO 12
      GOTO 1
C     Allow for "=". This sets the real equal to the previous one.
    2 IF (TEXT(N).EQ.'='.AND.I.NE.1) GOTO 11
C     Search for digits.
    3 DO 4 J=1,NDIG
C     Store the value of the digit.
      ND = J - 1
      IF (TEXT(N).EQ.DIGITS(J)) GOTO 5
C     Next digit.
    4 CONTINUE
C     Is it a minus sign ?
      IF (TEXT(N).EQ.'-') GOTO 7
C     Or decimal point.
      IF (TEXT(N).EQ.'.') GOTO 8
C     Or space separator.
      IF (TEXT(N).EQ.' ') GOTO 10
C     Character must be spurious.
      GOTO 13
C     Is digit before or after decimal point ?
    5 IF (NDP.EQ.0) GOTO 6
C     Add into real.
      VAL(I) = VAL(I) + FLOAT(ND) * 10.0 ** NDP
C     Increase the number of digits after the decimal point.
      NDP = NDP - 1
      GOTO 9
C     Add into real.
    6 VAL(I) = VAL(I) * 10.0 + FLOAT(ND)
      GOTO 9
C     If sign already found, then error.
    7 IF (NSGN.NE.0) GOTO 13
C     Set sign bit.
      SGN = - 1.0
      GOTO 9
C     If decimal point already found, then error.
    8 IF (NDP.NE.0) GOTO 13
C     Set decimal point.
      NDP = -1
C     Set the pointer to the next character.
    9 N = N + 1
C     If at end of string, assume blank characters follow.
      IF (N.EQ.NDIM2) GOTO 10
C     And loop.
      GOTO 3
C     Set the sign bit.
   10 VAL(I) = VAL(I) * SGN
      GOTO 12
C     "=" found. Set value equal to previous.
   11 VAL(I) = VAL(I-1)
C     Next real.
   12 CONTINUE
C     Return to calling subroutine.
      RETURN
C     Error condition.
   13 WRITE(IOW,14) N,TEXT
   14 FORMAT(/1X,'Spurious input at about column ',I2,' in line'
     +  /1X,75A1)
C     Abort.
      CALL ABORT
C     *******
      END
      SUBROUTINE DEFSPG
C     *******
C     Set default spacegroup from the lattice parameters.
C     Default held in array IDEF.
C     Use standard variable types for reals and integers.
C     NB Characters are explicitly defined.
      IMPLICIT REAL*4 (A-H,O-Z)
      IMPLICIT INTEGER*4 (I-N)
C     Input variables  : GA     AL     BE     A      B      C
C     Output variables : IDEF
C     Subroutines used :
C     Functions called :
      CHARACTER*1 ICH,ISPG,ISYM,IDEF
      COMMON/HKL1/ICH(24),ISPG(20),ISYM(6,3),IDEF(6)
      COMMON/HKL5/A,B,C,AL,BE,GA,COSA,AR2,BR2,CR2,BCR,CAR,ABR
C     Set default lattice type to primitive.
      IDEF(1) = 'P'
C     Set symmetry to blank.
      DO 1 I=2,6
    1 IDEF(I) = ' '
C     Is gamma not 90 ?
      IF (GA.NE.90.0) GOTO 4
C     Is alpha not 90 or beta not 90 ?
      IF (AL.NE.90.0.OR.BE.NE.90.0) GOTO 6
C     Is a not equal to b ?
      IF (A.NE.B) GOTO 3
C     Is b not equal to c ?
      IF (B.NE.C) GOTO 2
C     Set to general cubic spacegroup Pm3m.
      IDEF(2) = 'm'
      IDEF(3) = '3'
      IDEF(4) = 'm'
C     Return to calling subroutine.
      RETURN
C     Set to default tetragonal spacegroup P4/mmm.
    2 IDEF(2) = '4'
      IDEF(3) = '/'
      IDEF(4) = 'm'
      IDEF(5) = 'm'
      IDEF(6) = 'm'
C     Return to calling subroutine.
      RETURN
C     Set to default orthorhombic spacegroup Pmmm.
    3 IDEF(2) = 'm'
      IDEF(3) = 'm'
      IDEF(4) = 'm'
C     Return to calling subroutine.
      RETURN
C     Is a not equal to b ?
    4 IF (A.NE.B) GOTO 5
C     Is alpha not equal to 90 or beta not equal to 90 ?
      IF (AL.NE.90.0.OR.BE.NE.90.0) GOTO 7
C     If gamma not equal to 120, monoclinic (z-axis unique).
      IF (GA.NE.120.0) GOTO 8
C     Set to default hexagonal spacegroup P6/mmm.
      IDEF(2) = '6'
      IDEF(3) = '/'
      IDEF(4) = 'm'
      IDEF(5) = 'm'
      IDEF(6) = 'm'
C     Return to calling subroutine.
      RETURN
C     Are alpha and beta both 90 ?
    5 IF (AL.EQ.90.0.AND.BE.EQ.90.0) GOTO 8
      GOTO 9
C     Is alpha equal to 90 or beta equal to 90 ?
    6 IF (AL.EQ.90.0.OR.BE.EQ.90.0) GOTO 8
      GOTO 9
C     Is alpha not equal to gamma or beta not equal to gamma ?
C     Is a not equal to b or b not equal to c ?
    7 IF (AL.NE.GA.OR.BE.NE.GA.OR.A.NE.B.OR.B.NE.C) GOTO 9
C     Set to default rhombohedral spacegroup R-3m.
      IDEF(1) = 'R'
      IDEF(2) = '-'
      IDEF(3) = '3'
      IDEF(4) = 'm'
C     Return to calling subroutine.
      RETURN
C     Set to default monoclinic spacegroup P2/m.
    8 IDEF(2) = '2'
      IDEF(3) = '/'
      IDEF(4) = 'm'
C     Return to calling subroutine.
      RETURN
C     Set to default triclinic spacegroup P-1.
    9 IDEF(2) = '-'
      IDEF(3) = '1'
C     Return to calling subroutine.
      RETURN
C     *******
      END
      SUBROUTINE GETWAV
C     *******
C     Use standard variable types for reals and integers.
C     NB Characters are explicitly defined.
      IMPLICIT REAL*4 (A-H,O-Z)
      IMPLICIT INTEGER*4 (I-N)
C     Input variables  : IOW    DFWAV  DFTMX  DZP    IOR    LPT
C                        RADS2
C     Output variables : WAV    TMX    ZER    WAV2   DMIN
C     Subroutines used : EOFINP
C     Functions called :
      PARAMETER (NDIM1=4000,NDIM2=75)
      CHARACTER*1 TEXT
      COMMON/HKL3/IOR,IOW,LPT,IOUT
      COMMON/HKL4/PI,RADS,RADS2,NBIT,NREF,NTOT,NCYC,MAXHKL
      COMMON/HKL7/DMIN,WAV2,ZER,EMAX
      COMMON/HKL9/TEXT(NDIM2)
CMOD*
C     Extra variables and statements for PDPL routines
C     A.D. Murray (06-01-88)
      COMMON /PDPL2/ DFCELL(6),DFWAV,DZP
      COMMON /PDPL3/ DFCAOK,ENERGY
      LOGICAL        DFCAOK,ENERGY
      CHARACTER*80 LINEIN
      DOUBLE PRECISION VAL
      LOGICAL ERROR
      DIMENSION VAL(3)
      DATA NVAL/3/
CMOD*
C     Default wavelength and zeropoint set by DEFCEL
C     A.D. Murray (08-01-88)
C     Set default wavelength to Cu K-alpha.
C#    DATA DFWAV/1.54178/
C     Set default maximum two-theta value.
      DATA DFTMX/180.0/
C     Set default zero-point.
C#    DATA DZP/0.0/
CMOD*
C     End of modified section
C     Get wavelength and maximum two-theta limits.
C     For TOF diffaction, set the wavelength to twice the minimum
C     d-spacing required and set two-theta to default value.
CMOD*
C     Extra statement label
C     A.D. Murray (06-01-88)
 309  CONTINUE
      WRITE(IOW,1) DFWAV,DFTMX,DZP
    1 FORMAT(/1X,'Give wavelength, maximum 2-theta, and zero-point',
     +  1X,'(Obs = Calc + Z-P)'
     +  /1X,'(<CR> = ',F8.5,' Angstroms, ',F5.1,' degrees, &',
CMOD*
C     Use PDPL routine PROMPT instead of VAX $ 
C     A.D. Murray (07-01-88)
C#   +  1X,F7.3,' degrees)'/1X,'->  ',$)
     +  1X,F7.3,' degrees)')
      CALL PROMPT('Wavelength > ',13,IOW)
CMOD*
C     Use PDPL routines COMINT and DIGITS to get input
C     A.D. Murray (06-01-88)
C#    READ(IOR,2,END=7) TEXT
CMOD*
C     The following statement has been changed from;
C#  2 FORMAT(<NDIM2>A1)
C     because use of PARAMETER variables as edit descriptors in
C     FORMAT statements in not allowed in standard FORTRAN 77
C     A.D. Murray (04-01-88)
    2 FORMAT(75A1)
C#    CALL GETVAL(NVAL,VAL)
      CALL COMINT(0,LINEIN,IFLAG,'WAVELE')
      IF (IFLAG.EQ.2) GOTO 7
      IF (IFLAG.EQ.3) GOTO 309
      CALL DIGITS(LINEIN,VAL,3,1,KEND,NFND,ERROR,IOW)
      IF (ERROR) GOTO 309
CMOD*
C     End of modified section
      WAV = VAL(1)
      TMX = VAL(2)
      ZER = VAL(3)
C     Check wavelength and 2-theta max.
      IF (WAV.LT.0.0) GOTO 5
      IF (TMX.LT.0.0.OR.TMX.GT.180.0) GOTO 5
      IF (ZER.LT.-500.0.OR.ZER.GT.500.0) GOTO 5
C     If <CR>, then set defaults.
C     These are usually set Cu K-alpha radiation for X-rays,
C     but may be set to the wavelength most frequently used.
      IF (WAV.EQ.0.0) WAV = DFWAV
C     Set default maximum two-theta value required.
      IF (TMX.EQ.0.0) TMX = DFTMX
C     Set default zero-point.
CMOD*
C     Changed to allow non-zero default zerpoint to be set to 0
C     A.D. Murray (08-01-88)
C#    IF (ZER.EQ.0.0) ZER = DZP
      IF (ZER.EQ.0.0.AND.NFND.LT.3) ZER = DZP
C     Store defaults.
      DFWAV = WAV
      DFTMX = TMX
      DZP = ZER
      WRITE(LPT,3) WAV,TMX,ZER
    3 FORMAT(/'Wavelength ',F8.5,' A   Maximum 2-theta ',F8.3,
     +  1X,'deg   Zero-point ',F8.3,' deg')
C     Calculate minimum d-spacing.
      WAV2 = WAV / 2.0
C     Take into account zero-point - only relevant when very large !
      DMIN = WAV2 / SIN((TMX - ZER) * RADS2)
      WRITE(LPT,4) DMIN
    4 FORMAT(/'Minimum d-spacing is ',F8.5,' A')
C     Return to calling subroutine.
      RETURN
C     Give error message if error in wavelength.
    5 WRITE(IOW,6)
    6 FORMAT(/1X,'Impossible wavelength, two-theta limit, etc.')
CMOD* 
C     Allow another attempt if mistakes are made !!
C     A.D. Murray (06-01-88)
C     Abort to monitor !
C#    CALL ABORT
      GOTO 309
C     EOF on input, abort to monitor !
    7 CALL EOFINP
C     *******
      END
      SUBROUTINE GETSPG
C     *******
C     Use standard variable types for reals and integers.
C     NB Characters are explicitly defined.
      IMPLICIT REAL*4 (A-H,O-Z)
      IMPLICIT INTEGER*4 (I-N)
C     Input variables  : IOW    IDEF   IOR    ICH    LPT    NBIT
C     Output variables : ISPG   I      NCH    LTP    J      ISYM
C                        NS     IS     ID     IC     NC
C     Subroutines used : EXIT   EOFINP
C     Functions called : CHAR   ICHAR
      CHARACTER*1 ICH,ISPG,ISYM,IDEF
      COMMON/HKL1/ICH(24),ISPG(20),ISYM(6,3),IDEF(6)
      COMMON/HKL3/IOR,IOW,LPT,IOUT
      COMMON/HKL4/PI,RADS,RADS2,NBIT,NREF,NTOT,NCYC,MAXHKL
      COMMON/HKL6/LTP,LAU,NEX,IEX(32)
CMOD*
C     Extra variable for PDPL routines
      CHARACTER*80 LINEIN
CMOD*
C     Extra statement label
C     A.D. Murray (06-01-88)
 310  CONTINUE
      WRITE(IOW,1) IDEF
    1 FORMAT(/1X,'Give spacegroup (eg. P2(1)/m, Fm3m, P6(3)/mmc,',
     +  1X,'P-4, P1 etc.)'
     +  /1X,'(Default lattice is ',A1,' and symmetry is ',5A1,
CMOD*
C     Use PDPL routine PROMPT instead of VAX $ 
C     A.D. Murray (07-01-88)
C#   +  1X,'<CR> = default)'/1X,'->  ',$)
     +  1X,'<CR> = default)')
      CALL PROMPT('Spacegroup > ',13,IOW)
CMOD*
C     Use PDPL routine COMINT to get input
C     A.D. Murray (06-01-88)
C#    READ(IOR,2,END=42) ISPG
      CALL COMINT(0,LINEIN,IFLAG,'SPACEG')
      IF (IFLAG.EQ.2) GOTO 42
      IF (IFLAG.EQ.3) GOTO 310

        DO 311 IADM = 1,20
        ISPG(IADM) = LINEIN(IADM:IADM)
 311    CONTINUE

CMOD*
C     End of modified section
    2 FORMAT(20A1)
C     Check for blank line in input.
      DO 3 I=1,20
      IF (ISPG(I).NE.' ') GOTO 5
    3 CONTINUE
C     If <CR>, then set to default.
      DO 4 I=1,6
      ISPG(I) = IDEF(I)
    4 CONTINUE
C     Remove any spurious spaces from spacegroup symbol.
    5 NCH = 0
      DO 6 I=1,20
      IF (ISPG(I).EQ.' ') GOTO 6
C     Increment number of characters.
      NCH = NCH + 1
      ISPG(NCH) = ISPG(I)
    6 CONTINUE
C     NCH is number of characters in spacegroup symbol.
C     Set rest back to blank.
      IF (NCH.EQ.20) GOTO 8
      DO 7 I=(NCH+1),20
    7 ISPG(I) = ' '
C     Some terminals only have upper-case.
C     - and some people are either bad typists or blind !
CMOD*
C     Changed to use PDPL routines UPCASE and LOCASE
C     A.D. Murray (04-01-88)
C#  8 IF (ISPG(1).GE.'a'.AND.ISPG(1).LE.'z')
C#   +  ISPG(1) = CHAR(ICHAR(ISPG(1)) - 32)
C#    DO 9 I = 2,20
C#  9 IF (ISPG(I).GE.'A'.AND.ISPG(I).LE.'Z')
C#   +  ISPG(I) = CHAR(ICHAR(ISPG(I)) + 32)
    8 CALL UPCASE(ISPG(1),1)
      DO 9 I = 2,NCH
      CALL LOCASE(ISPG(I),1)
    9 CONTINUE
C     Determine symmetry constraints - decode spacegroup symbol.
C     Lattice types : P A B C F I R.
      DO 10 I=1,7
      LTP = I
      IF (ISPG(1).EQ.ICH(I)) GOTO 12
   10 CONTINUE
C     Give error message if unable to decode lattice type.
      WRITE(IOW,11) ISPG(1),ISPG
   11 FORMAT(/1X,'Error - "',A1,'" found at start of spacegroup',
     +  1X,'symbol ',20A1)
CMOD*
C     Allow another attempt if mistakes are made !!
C     A.D. Murray (06-01-88)
C     Abort
C#    CALL ABORT
      GOTO 310
C     If only lattice-centring specified, take default Laue type.
   12 IF (NCH.NE.1) GOTO 14
      DO 13 I=2,6
      ISPG(I) = IDEF(I)
   13 CONTINUE
C     Rotation axes with or without screw or inversion : 1 -1 2 (-2 = m)
C     2(1) 3 -3 3(1) 3(2) 4 -4 4(1) 4(2) 4(3) 6 -6 6(1) 6(2) 6(3) 6(4)
C     6(5).
C     Mirror and glide planes : m a b c n d.
C     First character is digit 1 2 3 4 6 or - or m a b c n d.
C     Set symbol array to blanks.
   14 DO 16 J=1,3
      DO 15 I=1,6
   15 ISYM(I,J) = ' '
   16 CONTINUE
C     Extract primary, secondary, and tertiary symbols. Check syntax.
C     NS is the symbol number, IS is its start position in the
C     array ISPG, and ID is a syntax indicator. ID = 0 shows new
C     symbol, ID = 1,2,3,4,5,6 shows previous characters as
C     "/","-","n(","n(m","n(m)", and "n" respectively where n and
C     m are digits such that n > m.
      NS = 1
      IS = 2
      ID = 0
C     IC is the pointer position in array ISPG.
      IC = 1
C     Get next character.
   17 IC = IC + 1
      DO 18 I=8,24
      NC = I
      IF (ISPG(IC).EQ.ICH(I)) GOTO 19
   18 CONTINUE
C     If spurious character, abort.
      GOTO 38
C     NC is the position of the character in the array ICH.
C     If digit, then jump.
   19 IF (NC.LE.13) GOTO 20
C     If alphabetic, then jump.
      IF (NC.LE.19) GOTO 24
      GOTO (25,26,27,28,29) NC - 19
C     Digit found - abort if ID is 1 or 4.
   20 GOTO (38,21,22,38,23,23) ID
C     ID is 0 or 2. Abort if digit is 5.
C     End of symbol if digit is 1.
   21 IF (NC.EQ.12) GOTO 38
C     Also abort if -2. Must use m.
      IF (ID.EQ.2.AND.NC.EQ.9) GOTO 38
      IF (NC.EQ.8) GOTO 31
      ID = 6
      GOTO 17
C     ID is 3. Abort if m >= n.
   22 IF (ISPG(IC-2).LE.ISPG(IC)) GOTO 38
      ID = 4
      GOTO 17
C     ID is 5 or 6 and start of next symbol is encountered.
   23 IC = IC - 1
      GOTO 31
C     Alphabetic found - abort if ID is 2, 3, or 4. End of symbol
C     if ID is 0 or 1 and start of next if ID is 5 or 6.
   24 IF (ID.LE.1) GOTO 31
      IF (ID.LE.4) GOTO 38
      IC = IC - 1
      GOTO 31
C     "-" found - next character must be digit.
C     If ID is 5 or 6, then next symbol.
   25 IF (ID.GE.5) GOTO 30
      IF (ID.NE.0) GOTO 38
      ID = 2
      GOTO 17
C     "/" found - next character must be alphabetic.
   26 IF (ID.LE.4) GOTO 38
      ID = 1
      GOTO 17
C     "(" found - must proceed digit m and be proceeded by digit n
C     where n > m. If ID not 6, then abort.
   27 IF (ID.NE.6) GOTO 38
C     Check that n not proceeded by "-".
      IF (ISPG(IC-2).EQ.'-') GOTO 38
      ID = 3
      GOTO 17
C     ")" found - If ID not 4, then abort.
   28 IF (ID.NE.4) GOTO 38
      ID = 5
      GOTO 17
C     " " found - If ID is 0, then end of input. If ID is 1 to 4,
C     then abort.
   29 IF (ID.EQ.0) GOTO 33
      IF (ID.LE.4) GOTO 38
   30 IC = IC - 1
C     Copy symbol into array ISYM.
   31 DO 32 I=IS,IC
      J = I + 1 - IS
   32 ISYM(J,NS) = ISPG(I)
C     If 3 symbols found, then quit parsing.
      IF (NS.EQ.3) GOTO 33
      NS = NS + 1
      IS = IC + 1
      ID = 0
C     Get next symbol.
      GOTO 17
C     Check rest of line is blank.
   33 IF (IC.EQ.20) GOTO 35
      DO 34 I=IC+1,20
      IC = I
      IF (ISPG(I).NE.' ') GOTO 38
   34 CONTINUE
   35 WRITE(LPT,36) ISPG
   36 FORMAT(/'Spacegroup ',20A1)
      WRITE(LPT,37) ISPG(1),((ISYM(I,J),I=1,6),J=1,3)
   37 FORMAT(/'Lattice ',A1,' 1st symbol ',6A1,' 2nd symbol ',6A1,
     +  1X,'3rd symbol ',6A1)
C     Return to calling subroutine.
      RETURN
C     Give error message if unable to decode spacegroup symbols.
   38 WRITE(IOW,39) ISPG(IC),ISPG(IC-1),ISPG
   39 FORMAT(/1X,'Error - "',A1,'" found after "',A1,'" in spacegroup',
     +  1X,'symbol ',20A1)
CMOD*
C     Allow another attempt if mistakes are made !!
C     A.D. Murray (06-01-88)
C     Abort.
C#    CALL ABORT
      GOTO 310
C     Give error message if impossible spacegroup.
   40 WRITE(IOW,41)
   41 FORMAT(/1X,'Unacceptable spacegroup')
CMOD*
C     Allow another attempt if mistakes are made !!
C     A.D. Murray (06-01-88)
C     Abort.
C#    CALL ABORT
      GOTO 310
C     EOF on input, abort to monitor !
   42 CALL EOFINP
C     *******
      END
      SUBROUTINE GETLAU(ERROR)
C     *******
C     Determine Laue type.
C     Use standard variable types for reals and integers.
C     NB Characters are explicitly defined.
      IMPLICIT REAL*4 (A-H,O-Z)
      IMPLICIT INTEGER*4 (I-N)
C     Input variables  : ISYM   ICH    IOW    ISPG   LPT    AL
C                        BE     GA
C     Output variables : NS     J      I      NA     NB     NC
C                        LAU
C     Subroutines used : EXIT
C     Functions called :
      CHARACTER*1 ICH,ISPG,ISYM,IDEF
      COMMON/HKL1/ICH(24),ISPG(20),ISYM(6,3),IDEF(6)
      COMMON/HKL3/IOR,IOW,LPT,IOUT
      COMMON/HKL5/A,B,C,AL,BE,GA,COSA,AR2,BR2,CR2,BCR,CAR,ABR
      COMMON/HKL6/LTP,LAU,NEX,IEX(32)
CMOD*
C     Extra variable - A.D. Murray (07-01-88)
      LOGICAL ERROR
      ERROR = .FALSE.
C     Get first alphanumeric from primary, secondary, and tertiary
C     symbols.
      NS = 0
    1 NS = NS + 1
      DO 3 J=1,6
      DO 2 I=8,19
      N = I
      IF (ISYM(J,NS).EQ.ICH(I)) GOTO 4
    2 CONTINUE
    3 CONTINUE
      N = 7
    4 GOTO (5,6,7) NS
    5 NA = N - 7
      GOTO 1
    6 NB = N - 7
      GOTO 1
    7 NC = N - 7
C     If first symbol is plane, jump.
      IF (NA.GT.6) GOTO 13
C     Crystal has rotation axis of order NA.
      GOTO (9,13,19,24) NA
C     Hexagonal case - Laue type 13 or 14.
C     Check 2nd and 3rd symbol for diads and/or planes.
      IF (NB.LT.7.AND.NB.NE.2) GOTO 8
      IF (NC.LT.7.AND.NC.NE.2) GOTO 27
      LAU = 14
C     Return to calling subroutine.
      RETURN
C     2nd symbol must be blank.
    8 IF (NB.NE.0) GOTO 27
      LAU = 13
C     Return to calling subroutine.
      RETURN
C     Triclinic or monoclinic cases - Laue type 1, 3, or 4.
C     If 2nd symbol is blank, then triclinic.
    9 IF (NB.EQ.0) GOTO 12
C     If 2nd symbol is monad axis, 3rd must be diad and/or plane.
      IF (NB.EQ.1) GOTO 11
      IF ((NB.EQ.2.OR.NB.GE.7).AND.NC.EQ.1) GOTO 10
      GOTO 27
   10 LAU = 3
C     Return to calling subroutine.
      RETURN
   11 IF (NC.NE.2.AND.NC.LT.7) GOTO 27
      LAU = 4
C     Return to calling subroutine.
      RETURN
C     Syntax check forces third symbol to be blank.
   12 LAU = 1
C     Return to calling subroutine.
      RETURN
C     Monoclinic, orthorhombic, or cubic cases -
C     Laue type 2, 3, 4, 5, 15, or 16.
C     If 2nd symbol is triad, then cubic.
   13 IF (NB.EQ.3) GOTO 18
C     If 2nd symbol diad or plane, then orthorhombic.
      IF (NB.LT.7.AND.NB.NE.2) GOTO 14
C     Check that 3rd symbol is diad and/or plane.
      IF (NC.LT.7.AND.NC.NE.2) GOTO 27
      LAU = 5
C     Return to calling subroutine.
      RETURN
C     If 2nd symbol is not monad or blank, abort.
   14 IF (NB.GT.1) GOTO 27
C     If 2nd symbol is blank, need to examine lattice parameters.
      IF (NB.EQ.1) GOTO 17
      IF (AL.EQ.90.0) GOTO 15
      LAU = 2
C     Return to calling subroutine.
      RETURN
   15 IF (BE.EQ.90.0) GOTO 16
      LAU = 3
C     Return to calling subroutine.
      RETURN
C     If alpha = beta = gamma = 90, give error.
   16 IF (GA.EQ.90.0) GOTO 29
      LAU = 4
C     Return to calling subroutine.
      RETURN
C     If 3rd symbol not monad, abort.
   17 IF (NC.NE.1) GOTO 27
      LAU = 2
C     Return to calling subroutine.
      RETURN
C     If 3rd symbol is blank, then type 15.
   18 IF (NC.NE.0) GOTO 26
      LAU = 15
C     Return to calling subroutine.
      RETURN
C     Trigonal case - Laue type 8, 9, 10, 11, or 12.
C     If lattice type is P, then only hexagonal axes.
   19 IF (LTP.EQ.1) GOTO 21
C     Lattice type is R. Examine lattice parameters.
      IF (GA.EQ.120.0) GOTO 21
C     If 2nd symbol blank, then type 8.
      IF (NB.EQ.0) GOTO 20
      IF (NB.GT.2.AND.NB.LT.7) GOTO 27
      IF (NC.GT.2.AND.NC.LT.7) GOTO 27
      IF (NB.EQ.NC.OR.(NB + NC).EQ.1) GOTO 27
      LAU = 9
C     Return to calling subroutine.
      RETURN
   20 LAU = 8
C     Return to calling subroutine.
      RETURN
C     If 2nd symbol blank, then type 10.
   21 IF (NB.EQ.0) GOTO 23
      IF (NB.GT.2.AND.NB.LT.7) GOTO 27
      IF (NB.EQ.1) GOTO 22
      IF (NC.GT.1) GOTO 27
      LAU = 11
C     Return to calling subroutine.
      RETURN
   22 IF (NC.GT.2.AND.NC.LT.7) GOTO 27
      LAU = 12
C     Return to calling subroutine.
      RETURN
   23 LAU = 10
C     Return to calling subroutine.
      RETURN
C     Tetragonal or cubic case - Laue type 6, 7, or 16.
C     If 2nd symbol is triad, then cubic.
   24 IF (NB.EQ.3) GOTO 26
C     2nd and 3rd symbols blank or diads and/or planes.
      IF (NB.NE.0) GOTO 25
      IF (NC.NE.0) GOTO 27
      LAU = 6
C     Return to calling subroutine.
      RETURN
   25 IF (NB.LT.7.AND.NB.NE.2) GOTO 27
      IF (NC.LT.7.AND.NC.NE.2) GOTO 27
      LAU = 7
C     Return to calling subroutine.
      RETURN
C     Check that 3rd symbol is diad or plane.
   26 IF (NC.LT.7.AND.NC.NE.2) GOTO 27
      LAU = 16
C     Return to calling subroutine.
      RETURN
C     Give error message if unable to decode spacegroup symbols.
   27 WRITE(IOW,28) ISPG(NS),ISPG(NS-1),ISPG
   28 FORMAT(/1X,'Error - "',A1,'" found after "',A1,'" in spacegroup',
     +  1X,'symbol ',20A1)
CMOD*
C     Allow another attempt if mistakes are made !!
C     A.D. Murray (06-01-88)
C     Abort.
C#    CALL ABORT
      ERROR = .TRUE.
      RETURN
C     Give error message if unable to find unique axis in monoclinic
C     system.
   29 WRITE(IOW,30)
   30 FORMAT(/1X,'Unable to find unique axis for monoclinic cell')
CMOD*
C     Allow another attempt if mistakes are made !!
C     A.D. Murray (06-01-88)
C     Abort.
C#    CALL ABORT
      ERROR = .TRUE.
      RETURN
C     *******
      END
      SUBROUTINE LAUEGP
C     *******
C     Writes out Laue group.
C     Use standard variable types for reals and integers.
C     NB Characters are explicitly defined.
      IMPLICIT REAL*4 (A-H,O-Z)
      IMPLICIT INTEGER*4 (I-N)
C     Input variables  : LAU    LPT    LTP
C     Output variables :
C     Subroutines used : ETRAP
C     Functions called :
      COMMON/HKL3/IOR,IOW,LPT,IOUT
      COMMON/HKL6/LTP,LAU,NEX,IEX(32)
C     16 values for LAU.
      GOTO (1,3,3,3,5,7,9,11,13,15,17,19,21,23,25,27) LAU
C     Error trap.
      CALL ETRAP
C     LAU = 1.
    1 WRITE(LPT,2)
    2 FORMAT(/'Laue class is -1')
C     Return to calling subroutine.
      RETURN
C     LAU = 2, 3, 4.
    3 WRITE(LPT,4)
    4 FORMAT(/'Laue class is 2/m')
C     Return to calling subroutine.
      RETURN
C     LAU = 5.
    5 WRITE(LPT,6)
    6 FORMAT(/'Laue class is mmm')
C     Return to calling subroutine.
      RETURN
C     LAU = 6.
    7 WRITE(LPT,8)
    8 FORMAT(/'Laue class is 4/m')
C     Return to calling subroutine.
      RETURN
C     LAU = 7.
    9 WRITE(LPT,10)
   10 FORMAT(/'Laue class is 4/mmm')
C     Return to calling subroutine.
      RETURN
C     LAU = 8.
   11 WRITE(LPT,12)
   12 FORMAT(/'Laue class is -3')
C     Return to calling subroutine.
      RETURN
C     LAU = 9.
   13 WRITE(LPT,14)
   14 FORMAT(/'Laue class is -3m')
C     Return to calling subroutine.
      RETURN
C     LAU = 10.
   15 WRITE(LPT,16)
   16 FORMAT(/'Laue class is -3')
C     Return to calling subroutine.
      RETURN
C     LAU = 11.
C     (For rhombohedral systems, it is -3m.)
   17 IF (LTP.EQ.7) GOTO 13
      WRITE(LPT,18)
   18 FORMAT(/'Laue class is -3m1')
C     Return to calling subroutine.
      RETURN
C     LAU = 12.
   19 WRITE(LPT,20)
   20 FORMAT(/'Laue class is -31m')
C     Return to calling subroutine.
      RETURN
C     LAU = 13.
   21 WRITE(LPT,22)
   22 FORMAT(/'Laue class is 6/m')
C     Return to calling subroutine.
      RETURN
C     LAU = 14.
   23 WRITE(LPT,24)
   24 FORMAT(/'Laue class is 6/mmm')
C     Return to calling subroutine.
      RETURN
C     LAU = 15.
   25 WRITE(LPT,26)
   26 FORMAT(/'Laue class is m3')
C     Return to calling subroutine.
      RETURN
C     LAU = 16.
   27 WRITE(LPT,28)
   28 FORMAT(/'Laue class is m3m')
C     Return to calling subroutine.
      RETURN
C     *******
      END
      SUBROUTINE CHECKC
C     *******
C     Check that spacegroup and lattice constants are compatible.
C     Use standard variable types for reals and integers.
C     NB Characters are explicitly defined.
      IMPLICIT REAL*4 (A-H,O-Z)
      IMPLICIT INTEGER*4 (I-N)
C     Input variables  : LAU    AL     BE     GA     A      B
C                        C      IOW    LPT
C     Output variables :
C     Subroutines used : EXIT
C     Functions called :
      COMMON/HKL3/IOR,IOW,LPT,IOUT
      COMMON/HKL5/A,B,C,AL,BE,GA,COSA,AR2,BR2,CR2,BCR,CAR,ABR
      COMMON/HKL6/LTP,LAU,NEX,IEX(32)
C     No conditions for triclinic case.
      IF (LAU.EQ.1) RETURN
      IF (LAU.EQ.2) GOTO 1
C     Alpha = beta = gamma for rhombohedral axes.
      IF (LAU.EQ.8.OR.LAU.EQ.9) GOTO 4
C     Is alpha 90 ?
      IF (AL.NE.90.0) GOTO 6
      IF (LAU.EQ.3) GOTO 2
C     Is beta 90 ?
    1 IF (BE.NE.90.0) GOTO 6
      IF (LAU.EQ.4) RETURN
C     Gamma = 120 for hexagonal axes.
      GOTO (3,3,3,3,3) LAU - 9
    2 IF (GA.NE.90.0) GOTO 6
C     a, b, and c may all differ for monoclinic and orthorhombic axes.
      IF (LAU.LE.5) RETURN
      GOTO 5
    3 IF (GA.NE.120.0) GOTO 6
      GOTO 5
    4 IF (AL.NE.BE.OR.BE.NE.GA) GOTO 6
    5 IF (A.NE.B) GOTO 6
C     b and c may differ for tetragonal and hexagonal axes.
      IF (LAU.LE.14.AND.LAU.NE.8.AND.LAU.NE.9) RETURN
C     b = c for cubic and rhombohedral axes.
      IF (B.NE.C) GOTO 6
C     Return to calling subroutine.
      RETURN
C     Give error message if cell constants and spacegroup are not
C     compatible.
    6 WRITE(IOW,7)
    7 FORMAT(/1X,'Cell constants incompatible with spacegroup')
C     Abort.
      CALL ABORT
C     *******
      END
      SUBROUTINE RHOMBO
C     *******
C     If rhombohedral, calculate alternative cell constants.
C     Use standard variable types for reals and integers.
C     NB Characters are explicitly defined.
      IMPLICIT REAL*4 (A-H,O-Z)
      IMPLICIT INTEGER*4 (I-N)
C     Input variables  : LTP    LAU    A      C      RADS   LPT
C                        COSA
C     Output variables : ARH    ALRH   AHE    CHE    ALHE   GAHE
C     Subroutines used :
C     Functions called : SQRT   ACOS
      COMMON/HKL3/IOR,IOW,LPT,IOUT
      COMMON/HKL4/PI,RADS,RADS2,NBIT,NREF,NTOT,NCYC,MAXHKL
      COMMON/HKL5/A,B,C,AL,BE,GA,COSA,AR2,BR2,CR2,BCR,CAR,ABR
      COMMON/HKL6/LTP,LAU,NEX,IEX(32)
C     If not rhombohedral, return to calling subroutine.
      IF (LTP.NE.7) RETURN
C     If rhombohedral axes specified, skip.
      IF (LAU.EQ.8.OR.LAU.EQ.9) GOTO 2
C     Hexagonal to rhombohedral conversion.
      ARH = SQRT(3.0 * A * A + C * C) / 3.0
      ALRH = ACOS((2.0 * C * C - 3.0 * A * A) / (2.0 * C * C +
     +  6.0 * A * A)) / RADS
      WRITE(LPT,1) ARH,ARH,ARH,ALRH,ALRH,ALRH
    1 FORMAT(/'Alternative cell constants for rhombohedral',
     +  1X,'systems'/3F9.4,1X,3F8.3)
C     Return to calling subroutine.
      RETURN
C     Rhombohedral to hexagonal conversion.
    2 AHE = A * SQRT(2.0 - 2.0 * COSA)
      CHE = A * SQRT(6.0 * COSA + 3.0)
      ALHE = 90.0
      GAHE = 120.0
      WRITE(LPT,1) AHE,AHE,CHE,ALHE,ALHE,GAHE
C     Return to calling subroutine.
      RETURN
C     *******
      END
      SUBROUTINE GENEXT
C     *******
C     Works out general extinctions.
C     Use standard variable types for reals and integers.
C     NB Characters are explicitly defined.
      IMPLICIT REAL*4 (A-H,O-Z)
      IMPLICIT INTEGER*4 (I-N)
C     Input variables  : LTP    LAU
C     Output variables : NEX    IEX
C     Subroutines used : ETRAP
C     Functions called :
      COMMON/HKL6/LTP,LAU,NEX,IEX(32)
C     Initialize number of extinctions.
      NEX = 0
C     7 lattice types.
      GOTO (1,2,3,4,5,6,7) LTP
C     Error trap.
      CALL ETRAP
C     Primitive. No extinction.
C     Return to calling subroutine.
    1 RETURN
C     A-face centred. x,1/2+y,1/2+z. hkl : k + l = 2n.
    2 NEX = NEX + 1
      IEX(NEX) = 1
C     Return to calling subroutine.
      RETURN
C     B-face centred. 1/2+x,y,1/2+z. hkl : h + l = 2n.
    3 NEX = NEX + 1
      IEX(NEX) = 2
C     Return to calling subroutine.
      RETURN
C     C-face centred. 1/2+x,1/2+y,z. hkl : h + k = 2n.
    4 NEX = NEX + 1
      IEX(NEX) = 3
C     Return to calling subroutine.
      RETURN
C     All (F) face centred. 1/2+x,1/2+y,z; 1/2+x,y,1/2+z;
C     x,1/2+y,1/2+z. hkl : h + k = 2n, h + l = 2n, (k + l = 2n).
    5 NEX = NEX + 1
      IEX(NEX) = 3
      NEX = NEX + 1
      IEX(NEX) = 2
C     Return to calling subroutine.
      RETURN
C     Body (I) centred. 1/2+x,1/2+y,1/2+z. hkl : h + k + l = 2n.
    6 NEX = NEX + 1
      IEX(NEX) = 4
C     Return to calling subroutine.
      RETURN
C     Rhombohedral with hexagonal axes, obverse setting chosen.
C     1/3+x,2/3+y,2/3+z; 2/3+x,1/3+y,1/3+z. hkl: - h + k + l = 3n.
C     If rhombohedral axes used, return to calling subroutine.
    7 IF (LAU.EQ.8.OR.LAU.EQ.9) RETURN
      NEX = NEX + 1
      IEX(NEX) = 5
C     Return to calling subroutine.
      RETURN
C     *******
      END
      SUBROUTINE SPEEXT
C     *******
C     Check lattice type against Laue type and work out special
C     extinctions in addition.
C     Use standard variable types for reals and integers.
C     NB Characters are explicitly defined.
      IMPLICIT REAL*4 (A-H,O-Z)
      IMPLICIT INTEGER*4 (I-N)
C     Input variables  :
C     Output variables :
C     Subroutines used :
C     Functions called :
      CHARACTER*1 ICH,ISPG,ISYM,IDEF
      COMMON/HKL1/ICH(24),ISPG(20),ISYM(6,3),IDEF(6)
      COMMON/HKL3/IOR,IOW,LPT,IOUT
      COMMON/HKL4/PI,RADS,RADS2,NBIT,NREF,NTOT,NCYC,MAXHKL
      COMMON/HKL6/LTP,LAU,NEX,IEX(32)
      GOTO (1,2,14,26,38,62,62,95,95) LAU
      GOTO (103,103,103,113,113,129,129) LAU - 9
C     Error trap.
      CALL ETRAP
C     Spacegroups P1 and P-1.
C     Lattice type R is not allowed. If A, B, C, F, or I centrings
C     are present, then the unit cell is in need of reduction.
    1 IF (LTP.EQ.7) GOTO 151
C     No point group extinctions possible.
C     Return to calling subroutine.
      RETURN
C     Spacegroups (x-axis unique) P2, P2(1) etc.
C     Lattice type R is not allowed. If A or F centrings
C     are present, then the unit cell is in need of reduction.
    2 IF (LTP.EQ.7) GOTO 151
C     Search for screw axis 2(1).
      DO 3 I=2,20
      IF (ISPG(I).EQ.ICH(22)) GOTO 4
    3 CONTINUE
      GOTO 5
C     2(1) axis. 1/2+x,-y,-z. h00 : h = 2n.
    4 NEX = NEX + 1
      IEX(NEX) = 6
C     Search for glide planes. Reject if a or d. Ignore if m.
    5 DO 6 I=2,20
      J = I
      IF (ISPG(I).GE.'a'.AND.ISPG(I).LE.'z') GOTO 7
    6 CONTINUE
C     Return to calling subroutine.
      RETURN
    7 DO 8 I=14,19
      NA = I - 13
      IF (ISPG(J).EQ.ICH(I)) GOTO 9
    8 CONTINUE
C     Error trap.
      CALL ETRAP
C     Found m, a, b, c, n, or d.
    9 GOTO (11,153,10,12,13,153) NA
C     Error trap.
      CALL ETRAP
C     b plane. -x,1/2+y,z. 0kl : k = 2n.
   10 NEX = NEX + 1
      IEX(NEX) = 10
C     Return to calling subroutine.
   11 RETURN
C     c plane. -x,y,1/2+z. 0kl : l = 2n.
   12 NEX = NEX + 1
      IEX(NEX) = 14
C     Return to calling subroutine.
      RETURN
C     n plane. -x,1/2+y,1/2+z. 0kl : k + l = 2n.
   13 NEX = NEX + 1
      IEX(NEX) = 17
C     Return to calling subroutine.
      RETURN
C     Spacegroups (y-axis unique) P2, P2(1) etc.
C     Lattice type R is not allowed. If B or F centrings
C     are present, then the unit cell is in need of reduction.
   14 IF (LTP.EQ.7) GOTO 151
C     Search for screw axis 2(1).
      DO 15 I=2,20
      IF (ISPG(I).EQ.ICH(22)) GOTO 16
   15 CONTINUE
      GOTO 17
C     2(1) axis. -x,1/2+y,-z. 0k0 : k = 2n.
   16 NEX = NEX + 1
      IEX(NEX) = 7
C     Search for glide planes. Reject if b or d. Ignore if m.
   17 DO 18 I=2,20
      J = I
      IF (ISPG(I).GE.'a'.AND.ISPG(I).LE.'z') GOTO 19
   18 CONTINUE
C     Return to calling subroutine.
      RETURN
   19 DO 20 I=14,19
      NA = I - 13
      IF (ISPG(J).EQ.ICH(I)) GOTO 21
   20 CONTINUE
C     Error trap.
      CALL ETRAP
C     Found m, a, b, c, n, or d.
   21 GOTO (23,22,153,24,25,153) NA
C     Error trap.
      CALL ETRAP
C     a plane. 1/2+x,-y,z. h0l : h = 2n.
   22 NEX = NEX + 1
      IEX(NEX) = 12
C     Return to calling subroutine.
   23 RETURN
C     c plane. x,-y,1/2+z. h0l : l = 2n.
   24 NEX = NEX + 1
      IEX(NEX) = 11
C     Return to calling subroutine.
      RETURN
C     n plane. 1/2+x,-y,1/2+z. h0l : h + l = 2n.
   25 NEX = NEX + 1
      IEX(NEX) = 16
C     Return to calling subroutine.
      RETURN
C     Spacegroups (z-axis unique) P2, P2(1) etc.
C     Lattice type R is not allowed. If C or F centrings
C     are present, then the unit cell is in need of reduction.
   26 IF (LTP.EQ.7) GOTO 151
C     Search for screw axis 2(1).
      DO 27 I=2,20
      IF (ISPG(I).EQ.ICH(22)) GOTO 28
   27 CONTINUE
      GOTO 29
C     2(1) axis. -x,-y,1/2+z. 00l : l = 2n.
   28 NEX = NEX + 1
      IEX(NEX) = 8
C     Search for glide planes. Reject if c or d. Ignore if m.
   29 DO 30 I=2,20
      J = I
      IF (ISPG(I).GE.'a'.AND.ISPG(I).LE.'z') GOTO 31
   30 CONTINUE
C     Return to calling subroutine.
      RETURN
   31 DO 32 I=14,19
      NA = I - 13
      IF (ISPG(J).EQ.ICH(I)) GOTO 33
   32 CONTINUE
C     Error trap.
      CALL ETRAP
C     Found m, a, b, c, n, or d.
   33 GOTO (35,34,36,153,37,153) NA
C     Error trap.
      CALL ETRAP
C     a plane. 1/2+x,y,-z. hk0 : h = 2n.
   34 NEX = NEX + 1
      IEX(NEX) = 9
C     Return to calling subroutine.
   35 RETURN
C     b plane. x,1/2+y,-z. hk0 : k = 2n.
   36 NEX = NEX + 1
      IEX(NEX) = 13
C     Return to calling subroutine.
      RETURN
C     n plane. 1/2+x,1/2+y,-z. hk0 : h + k = 2n.
   37 NEX = NEX + 1
      IEX(NEX) = 15
C     Return to calling subroutine.
      RETURN
C     Spacegroups P222 etc.
C     Lattice type R is not allowed.
   38 IF (LTP.EQ.7) GOTO 151
C     Do for each symbol.
      DO 61 J=1,3
C     Search for screw axis 2(1).
      DO 39 I=1,6
      IF (ISYM(I,J).EQ.ICH(22)) GOTO 40
   39 CONTINUE
      GOTO 41
C     2(1) axis along
C     x : 1/2+x,-y,-z. h00 : h = 2n.
C     y : -x,1/2+y,-z. 0k0 : k = 2n.
C     z : -x,-y,1/2+z. 00l : l = 2n.
   40 NEX = NEX + 1
      IEX(NEX) = J + 5
C     Search for glide planes. Reject if a, b, or c lies along axis.
C     Ignore if m.
   41 DO 42 I=1,6
      K = I
      IF (ISYM(I,J).GE.'a'.AND.ISYM(I,J).LE.'z') GOTO 43
   42 CONTINUE
      GOTO 61
   43 DO 44 I=14,19
      NA = I - 13
      IF (ISYM(K,J).EQ.ICH(I)) GOTO 45
   44 CONTINUE
C     Error trap.
      CALL ETRAP
C     Refering to x, y, or z axes.
   45 GOTO (46,51,56) J
C     Refers to x axis.
C     Found m, a, b, c, n, or d.
   46 GOTO (61,153,47,48,49,50) NA
C     Error trap.
      CALL ETRAP
C     b plane. -x,1/2+y,z. 0kl : k = 2n.
   47 NEX = NEX + 1
      IEX(NEX) = 10
      GOTO 61
C     c plane. -x,y,1/2+z. 0kl : l = 2n.
   48 NEX = NEX + 1
      IEX(NEX) = 14
      GOTO 61
C     n plane. -x,1/2+y,1/2+z. 0kl : k + l = 2n.
   49 NEX = NEX + 1
      IEX(NEX) = 17
      GOTO 61
C     d plane. -x,1/4+y,1/4+z. 0kl : k + l = 4n.
   50 NEX = NEX + 1
      IEX(NEX) = 20
      GOTO 61
C     Refers to y axis.
C     Found m, a, b, c, n, or d.
   51 GOTO (61,52,153,53,54,55) NA
C     Error trap.
      CALL ETRAP
C     a plane. 1/2+x,-y,z. h0l : h = 2n.
   52 NEX = NEX + 1
      IEX(NEX) = 12
      GOTO 61
C     c plane. x,-y,1/2+z. h0l : l = 2n.
   53 NEX = NEX + 1
      IEX(NEX) = 11
      GOTO 61
C     n plane. 1/2+x,-y,1/2+z. h0l : h + l = 2n.
   54 NEX = NEX + 1
      IEX(NEX) = 16
      GOTO 61
C     d plane. 1/4+x,-y,1/4+z. h0l : h + l = 4n.
   55 NEX = NEX + 1
      IEX(NEX) = 19
      GOTO 61
C     Refers to z axis.
C     Found m, a, b, c, n, or d.
   56 GOTO (61,57,58,153,59,60) NA
C     Error trap.
      CALL ETRAP
C     a plane. 1/2+x,y,-z. hk0 : h = 2n.
   57 NEX = NEX + 1
      IEX(NEX) = 9
      GOTO 61
C     b plane. x,1/2+y,-z. hk0 : k = 2n.
   58 NEX = NEX + 1
      IEX(NEX) = 13
      GOTO 61
C     n plane. 1/2+x,1/2+y,-z. hk0 : h + k = 2n.
   59 NEX = NEX + 1
      IEX(NEX) = 15
      GOTO 61
C     d plane. 1/4+x,1/4+y,-z. hk0 : h + k = 4n.
   60 NEX = NEX + 1
      IEX(NEX) = 18
   61 CONTINUE
C     Return to calling subroutine.
      RETURN
C     Spacegroups P4 etc. & P422 etc.
C     Lattice types A, B, & R are not allowed.
   62 IF (LTP.EQ.2.OR.LTP.EQ.3.OR.LTP.EQ.7) GOTO 151
C     Search for screw axes 4(1), 4(2), or 4(3).
      DO 63 I=1,6
      K = I
      IF (ISYM(I,1).EQ.ICH(22)) GOTO 64
   63 CONTINUE
      GOTO 68
C     4(1), 4(2), or 4(3) axis along z.
   64 NEX = NEX + 1
      K = K + 1
C     Look at next character to distinguish between 4(1), 4(2),
C     and 4(3).
      DO 65 I=8,10
      NA = I - 8
      IF (ISYM(K,1).EQ.ICH(I)) GOTO 66
   65 CONTINUE
C     Error trap.
      CALL ETRAP
   66 IF (NA.EQ.1) GOTO 67
C     4(1) or 4(3) axis. -y,x,1/4+z etc. 00l : l = 4n.
      IEX(NEX) = 21
      GOTO 68
C     4(2) axis. -y,x,1/2+z. 00l : l = 2n.
   67 IEX(NEX) = 8
C     Search for glide planes. Reject if c, ignore if m.
   68 DO 69 I=1,6
      K = I
      IF (ISYM(I,1).GE.'a'.AND.ISYM(I,1).LE.'z') GOTO 70
   69 CONTINUE
      GOTO 76
   70 DO 71 I=14,19
      NA = I - 13
      IF (ISYM(K,1).EQ.ICH(I)) GOTO 72
   71 CONTINUE
C     Error trap.
      CALL ETRAP
C     Refers to z axis.
C     Found m, a, b, c, n, or d.
   72 GOTO (76,73,73,153,74,75) NA
C     Error trap.
      CALL ETRAP
C     a & b planes. 1/2+x,y,-z. hk0 : h = 2n, k = 2n.
   73 NEX = NEX + 1
      IEX(NEX) = 9
      NEX = NEX + 1
      IEX(NEX) = 13
      GOTO 76
C     n plane. 1/2+x,1/2+y,-z. hk0 : h + k = 2n.
   74 NEX = NEX + 1
      IEX(NEX) = 15
      GOTO 76
C     d plane. 1/4+x,1/4+y,1/4-z. hk0 : h + k = 4n.
   75 NEX = NEX + 1
      IEX(NEX) = 18
   76 IF (LAU.EQ.6) RETURN
C     Spacegroups P422 etc.
C     Search secondary and tertiary symbols.
C     These apply to [100] and [110] directions respectively.
      DO 94 J=2,3
C     Search for screw axis 2(1).
      DO 77 I=1,6
      IF (ISYM(I,J).EQ.ICH(22)) GOTO 78
   77 CONTINUE
      GOTO 80
   78 IF (J.EQ.3) GOTO 79
C     2(1) axis along x. 1/2+x,-y,-z. h00 : h = 2n.
      NEX = NEX + 1
      IEX(NEX) = 6
      GOTO 80
C     2(1) axis along [110]. 1/4+y,1/4+x,-z. hh0 : h = 2n.
   79 NEX = NEX + 1
      IEX(NEX) = 24
C     Search for glide planes. Ignore if m.
   80 DO 81 I=1,6
      K = I
      IF (ISYM(I,J).GE.'a'.AND.ISYM(I,J).LE.'z') GOTO 82
   81 CONTINUE
      GOTO 94
   82 DO 83 I=14,19
      NA = I - 13
      IF (ISYM(K,J).EQ.ICH(I)) GOTO 84
   83 CONTINUE
C     Error trap.
      CALL ETRAP
   84 IF (J.EQ.3) GOTO 89
C     Refers to x & y axes.
C     Found m, a, b, c, n, or d.
      GOTO (94,85,85,86,87,88) NA
C     Error trap.
      CALL ETRAP
C     a & b planes. 1/2+x,-y,z. h0l : h = 2n.
C     (Redundant 0kl : k = 2n.)
   85 NEX = NEX + 1
      IEX(NEX) = 12
      GOTO 94
C     c plane. x,-y,1/2+z. h0l : l = 2n.
C     (Redundant 0kl : l = 2n.)
   86 NEX = NEX + 1
      IEX(NEX) = 11
      GOTO 94
C     n plane. 1/2+x,-y,1/2+z. h0l : h + l = 2n.
C     (Redundant 0kl : k + l = 2n.)
   87 NEX = NEX + 1
      IEX(NEX) = 16
      GOTO 94
C     d plane. 1/4+x,-y,1/4+z. h0l : h + l = 4n.
C     (Redundant 0kl : k + l = 4n.)
   88 NEX = NEX + 1
      IEX(NEX) = 19
      GOTO 94
C     Refers to [110] direction.
C     Found m, a, b, c, n, or d.
   89 GOTO (94,90,90,91,92,93) NA
C     Error trap.
      CALL ETRAP
C     a plane. 1/2+y,1/2+x,z. hhl : h = 2n.
   90 NEX = NEX + 1
      IEX(NEX) = 23
      GOTO 94
C     c plane. y,x,1/2+z. hhl : l = 2n.
   91 NEX = NEX + 1
      IEX(NEX) = 22
      GOTO 94
C     n plane.
   92 NEX = NEX + 1
      IEX(NEX) = 25
      GOTO 94
C     d plane.
   93 NEX = NEX + 1
      IEX(NEX) = 26
   94 CONTINUE
C     Return to calling subroutine.
      RETURN
C     Spacegroups R3 etc. & R32 etc.
C     Lattice type must be R.
   95 IF (LTP.NE.7) GOTO 151
C     Check that primary symbol has no screw axes or planes.
      DO 96 I=1,6
      IF (ISYM(I,1).EQ.ICH(22).OR.ISYM(I,1).EQ.'/') GOTO 153
   96 CONTINUE
C     No point group extinctions possible for R3 etc.
      IF (LAU.EQ.8) RETURN
C     Search secondary symbol for glide planes. Reject if a, b, n,
C     or d. Ignore if m.
      DO 97 I=1,6
      K = I
      IF (ISYM(I,2).GE.'a'.AND.ISYM(I,2).LE.'z') GOTO 98
   97 CONTINUE
C     Return to calling subroutine.
      RETURN
   98 DO 99 I=14,19
      NA = I - 13
      IF (ISYM(K,2).EQ.ICH(I)) GOTO 100
   99 CONTINUE
C     Error trap.
      CALL ETRAP
C     Found m, a, b, c, n, or d.
  100 GOTO (102,153,153,101,153,153) NA
C     Error trap.
      CALL ETRAP
C     c plane. 1/2+y,1/2+x,1/2+z. hhl : l = 2n, hkk : h = 2n.
  101 NEX = NEX + 1
      IEX(NEX) = 22
      NEX = NEX + 1
      IEX(NEX) = 29
C     Return to calling subroutine.
  102 RETURN
C     Spacegroups P3 etc, P321 etc, & P312 etc.
C     or R3 & R32 with hexagonal axes.
C     Lattice type must be P or R.
  103 IF (LTP.NE.1.AND.LTP.NE.7) GOTO 151
C     Search for screw axis 3(1) or 3(2).
      IF (ISYM(2,1).EQ.ICH(22)) GOTO 104
      GOTO 105
C     3(1) or 3(2) axis. 00l : l = 3n.
  104 NEX = NEX + 1
      IEX(NEX) = 27
  105 CONTINUE
C     No point group extinctions possible for P3 etc. and R3 etc.
      IF (LAU.EQ.10) RETURN
C     Spacegroups P3c1, P31c, & R3c.
C     Search secondary or tertiary symbol for glide planes.
C     Reject if a, b, n, or d. Ignore if m.
      J = LAU - 9
      DO 106 I=1,6
      K = I
      IF (ISYM(I,J).GE.'a'.AND.ISYM(I,J).LE.'z') GOTO 107
  106 CONTINUE
C     Return to calling subroutine.
      RETURN
  107 DO 108 I=14,19
      NA = I - 13
      IF (ISYM(K,J).EQ.ICH(I)) GOTO 109
  108 CONTINUE
C     Error trap.
      CALL ETRAP
C     Found m, a, b, c, n, or d.
  109 GOTO (111,153,153,110,153,153) NA
C     Error trap.
      CALL ETRAP
  110 IF (LAU.EQ.12) GOTO 112
C     Secondary symbol.
C     c plane.  h0l : l = 2n & 0kl : l = 2n.
      NEX = NEX + 1
      IEX(NEX) = 11
      NEX = NEX + 1
      IEX(NEX) = 14
C     Return to calling subroutine.
  111 RETURN
C     Tertiary symbol.
C     c plane. hhl : l = 2n & 2h-hl : l = 2n.
  112 NEX = NEX + 1
      IEX(NEX) = 22
      NEX = NEX + 1
      IEX(NEX) = 31
C     Return to calling subroutine.
      RETURN
C     Spacegroups P6 etc. & P622 etc.
C     Lattice type must be P.
  113 IF (LTP.NE.1) GOTO 151
C     Search for screw axis 6(1), 6(2), 6(3), 6(4), 6(5).
      DO 114 I=1,6
      K = I
      IF (ISYM(I,1).EQ.ICH(22)) GOTO 115
  114 CONTINUE
      GOTO 121
  115 NEX = NEX + 1
      K = K + 1
C     Look at next character to distinguish between 6(1), 6(2),
C     6(3), 6(4), and 6(5).
      DO 116 I=8,12
      NA = I - 7
      IF (ISYM(K,1).EQ.ICH(I)) GOTO 117
  116 CONTINUE
C     Error trap.
      CALL ETRAP
  117 GOTO (118,119,120,119,118) NA
C     Error trap.
      CALL ETRAP
  118 IEX(NEX) = 28
      GOTO 121
  119 IEX(NEX) = 27
      GOTO 121
  120 IEX(NEX) = 8
  121 IF (LAU.EQ.13) RETURN
C     Spacegroups P622 etc.
C     Search secondary or tertiary symbol for glide planes.
C     Reject if a, b, n, or d. Ignore if m.
      DO 128 J=2,3
      DO 122 I=1,6
      K = I
      IF (ISYM(I,J).GE.'a'.AND.ISYM(I,J).LE.'z') GOTO 123
  122 CONTINUE
      GOTO 128
  123 DO 124 I=14,19
      NA = I - 13
      IF (ISYM(K,J).EQ.ICH(I)) GOTO 125
  124 CONTINUE
C     Error trap.
      CALL ETRAP
C     Found m, a, b, c, n, or d.
  125 GOTO (128,153,153,126,153,153) NA
C     Error trap.
      CALL ETRAP
  126 IF (J.EQ.3) GOTO 127
C     Secondary symbol.
C     c plane. h0l : l = 2n.
      NEX = NEX + 1
      IEX(NEX) = 11
      GOTO 128
C     Tertiary symbol.
C     c plane. hhl : l = 2n.
  127 NEX = NEX + 1
      IEX(NEX) = 22
  128 CONTINUE
C     Return to calling subroutine.
      RETURN
C     Spacegroups P23 etc. & P432 etc.
C     Lattice type must be P, F, or I.
  129 IF (LTP.NE.1.AND.LTP.NE.5.AND.LTP.NE.6) GOTO 151
C     First symbol. Is first character 2 or 4 ?
      IF (ISYM(1,1).EQ.'2') GOTO 130
      IF (ISYM(1,1).EQ.'4') GOTO 132
C     Skip if plane found.
      GOTO 135
C     Check for screw axis 2(1).
  130 IF (ISYM(2,1).EQ.ICH(22)) GOTO 131
C     Skip if 2 axis found.
      GOTO 135
C     2(1) axis. 1/2+x,1/2+y,-z. h00 : h = 2n.
C     (Redundant 0k0 : k = 2n, 00l : l = 2n.)
  131 NEX = NEX + 1
      IEX(NEX) = 6
      GOTO 135
C     Check for screw axes 4(1), 4(2), or 4(3).
  132 IF (ISYM(2,1).EQ.ICH(22)) GOTO 133
C     Skip if 4 or -4 axes found.
      GOTO 135
C     Look at next character to distinguish between 4(1), 4(2),
C     and 4(3).
  133 NEX = NEX + 1
      IF (ISYM(3,1).EQ.'2') GOTO 134
C     4(1) or 4(3) axis. 1/4+x,1/4-z,1/4+y. h00 : h = 4n.
C     (Redundant 0k0 : k = 4n, 00l : l = 4n.)
      IEX(NEX) = 30
      GOTO 135
C     4(2) axis. 1/2+x,-z,y. h00 : h = 2n.
C     (Redundant 0k0 : k = 2n, 00l : l = 2n.)
  134 IEX(NEX) = 6
C     Search for glide planes.
  135 DO 136 I=1,6
      K = I
      IF (ISYM(I,1).GE.'a'.AND.ISYM(I,1).LE.'z') GOTO 137
  136 CONTINUE
      GOTO 143
  137 DO 138 I=14,19
      NA = I - 13
      IF (ISYM(K,1).EQ.ICH(I)) GOTO 139
  138 CONTINUE
C     Error trap.
      CALL ETRAP
C     Found m, a, b, c, n, or d.
  139 GOTO (143,140,140,140,141,142) NA
C     Error trap.
      CALL ETRAP
C     a (or b or c) plane. 1/2+x,y,-z. hk0 : h = 2n (cyc).
  140 NEX = NEX + 1
      IEX(NEX) = 9
      IF (LAU.EQ.16) GOTO 143
C     h0l : l = 2n (Spacegroups P23 etc. only)
      NEX = NEX + 1
      IEX(NEX) = 11
      GOTO 143
C     n plane. 1/2+x,1/2+y,1/2-z. hk0 : h + k = 2n (cyc).
  141 NEX = NEX + 1
      IEX(NEX) = 15
      IF (LAU.EQ.16) GOTO 143
C     h0l : h + l = 2n (Spacegroups P23 etc. only)
      NEX = NEX + 1
      IEX(NEX) = 16
      GOTO 143
C     d plane. 1/4+x,1/4+y,1/4-z. hk0 : h + k = 4n (cyc).
  142 NEX = NEX + 1
      IEX(NEX) = 18
      IF (LAU.EQ.16) GOTO 143
C     h0l : h + l = 4n (Spacegroups P23 etc. only)
      NEX = NEX + 1
      IEX(NEX) = 19
C     Spacegroups P432 etc.
  143 IF (LAU.EQ.15) RETURN
C     Tertiary symbol.
C     Search for glide planes.
      DO 144 I=1,6
      K = I
      IF (ISYM(I,3).GE.'a'.AND.ISYM(I,3).LE.'z') GOTO 145
  144 CONTINUE
C     Return to calling subroutine.
      RETURN
  145 DO 146 I=14,19
      NA = I - 13
      IF (ISYM(K,3).EQ.ICH(I)) GOTO 147
  146 CONTINUE
C     Error trap.
      CALL ETRAP
  147 ISO = ISO + 1
C     Found m, a, b, c, n, or d.
      GOTO (149,148,148,148,148,150) NA
C     Error trap.
      CALL ETRAP
C     a, b, and c planes. y,x,1/2+z. hhl : l = 2n, hkk : h = 2n.
C     (Redundant hkh : k = 2n.)
C     Also n plane.
  148 NEX = NEX + 1
      IEX(NEX) = 22
      NEX = NEX + 1
      IEX(NEX) = 29
C     Return to calling subroutine.
  149 RETURN
C     d plane. hhl : 2h + l = 4n, hkk : h + 2k = 4n.
C     (Redundant hkh : 2h + k = 4n.)
  150 NEX = NEX + 1
      IEX(NEX) = 26
      NEX = NEX + 1
      IEX(NEX) = 32
C     Return to calling subroutine.
      RETURN
C     Give error message if lattice and Laue type incompatible.
  151 WRITE(IOW,152)
  152 FORMAT(/1X,'Point group incompatible with lattice type')
C     Abort.
      CALL ABORT
C     Give error message if impossible spacegroup.
  153 WRITE(IOW,154)
  154 FORMAT(/1X,'Uninterpretable spacegroup')
C     Abort.
      CALL ABORT
C     *******
      END
      SUBROUTINE EXTINC
C     *******
C     Writes out extinctions conditions.
C     Use standard variable types for reals and integers.
C     NB Characters are explicitly defined.
      IMPLICIT REAL*4 (A-H,O-Z)
      IMPLICIT INTEGER*4 (I-N)
C     Input variables  :
C     Output variables :
C     Subroutines used :
C     Functions called :
      CHARACTER*1 ICH,ISPG,ISYM,IDEF
      COMMON/HKL1/ICH(24),ISPG(20),ISYM(6,3),IDEF(6)
      COMMON/HKL3/IOR,IOW,LPT,IOUT
      COMMON/HKL4/PI,RADS,RADS2,NBIT,NREF,NTOT,NCYC,MAXHKL
      COMMON/HKL6/LTP,LAU,NEX,IEX(32)
C     If no extinctions, skip.
      IF (NEX.EQ.0) GOTO 67
      WRITE(LPT,1)
    1 FORMAT(/'Extinction conditions')
C     Do for each extinction condition.
      DO 66 I=1,NEX
C     32 extinction possible here.
      GOTO (2,4,6,8,10,12,14,16,18,20,22,24,26,28,30,32,34,36,38,40,42,4
     +4,46,48,50,52,54,56,58,60,62,64) IEX(I)
C     Error trap.
      CALL ETRAP
C     A-face centred. hkl : k + l = 2n.
    2 WRITE(LPT,3)
    3 FORMAT(2X,'hkl : k + l = 2n')
      GOTO 66
C     B-face centred. hkl : h + l = 2n.
    4 WRITE(LPT,5)
    5 FORMAT(2X,'hkl : h + l = 2n')
      GOTO 66
    6 WRITE(LPT,7)
    7 FORMAT(2X,'hkl : h + k = 2n')
      GOTO 66
    8 WRITE(LPT,9)
    9 FORMAT(2X,'hkl : h + k + l = 2n')
      GOTO 66
   10 WRITE(LPT,11)
   11 FORMAT(2X,'hkl : - h + k + l = 3n')
      GOTO 66
   12 WRITE(LPT,13)
   13 FORMAT(2X,'h00 : h = 2n')
      GOTO 66
   14 WRITE(LPT,15)
   15 FORMAT(2X,'0k0 : k = 2n')
      GOTO 66
   16 WRITE(LPT,17)
   17 FORMAT(2X,'00l : l = 2n')
      GOTO 66
   18 WRITE(LPT,19)
   19 FORMAT(2X,'hk0 : h = 2n')
      GOTO 66
   20 WRITE(LPT,21)
   21 FORMAT(2X,'0kl : k = 2n')
      GOTO 66
   22 WRITE(LPT,23)
   23 FORMAT(2X,'h0l : l = 2n')
      GOTO 66
   24 WRITE(LPT,25)
   25 FORMAT(2X,'h0l : h = 2n')
      GOTO 66
   26 WRITE(LPT,27)
   27 FORMAT(2X,'hk0 : k = 2n')
      GOTO 66
   28 WRITE(LPT,29)
   29 FORMAT(2X,'0kl : l = 2n')
      GOTO 66
   30 WRITE(LPT,31)
   31 FORMAT(2X,'hk0 : h + k = 2n')
      GOTO 66
   32 WRITE(LPT,33)
   33 FORMAT(2X,'h0l : h + l = 2n')
      GOTO 66
   34 WRITE(LPT,35)
   35 FORMAT(2X,'0kl : k + l = 2n')
      GOTO 66
   36 WRITE(LPT,37)
   37 FORMAT(2X,'hk0 : h + k = 4n')
      GOTO 66
   38 WRITE(LPT,39)
   39 FORMAT(2X,'h0l : h + l = 4n')
      GOTO 66
   40 WRITE(LPT,41)
   41 FORMAT(2X,'0kl : k + l = 4n')
      GOTO 66
   42 WRITE(LPT,43)
   43 FORMAT(2X,'00l : l = 4n')
      GOTO 66
   44 WRITE(LPT,45)
   45 FORMAT(2X,'hhl : l = 2n')
      GOTO 66
   46 WRITE(LPT,47)
   47 FORMAT(2X,'hhl : h = 2n')
      GOTO 66
   48 WRITE(LPT,49)
   49 FORMAT(2X,'hh0 : h = 2n')
      GOTO 66
   50 WRITE(LPT,51)
   51 FORMAT(2X,'hhl : h + l = 2n')
      GOTO 66
   52 WRITE(LPT,53)
   53 FORMAT(2X,'hhl : 2h + l = 4n')
      GOTO 66
   54 WRITE(LPT,55)
   55 FORMAT(2X,'00l : l = 3n')
      GOTO 66
   56 WRITE(LPT,57)
   57 FORMAT(2X,'00l : l = 6n')
      GOTO 66
   58 WRITE(LPT,59)
   59 FORMAT(2X,'hkk : h = 2n')
      GOTO 66
   60 WRITE(LPT,61)
   61 FORMAT(2X,'h00 : h = 4n')
      GOTO 66
   62 WRITE(LPT,63)
   63 FORMAT(2X,'2h-hl : l = 2n')
      GOTO 66
   64 WRITE(LPT,65)
   65 FORMAT(2X,'hkk : h + 2k = 4n')
C     Next extinction condition.
   66 CONTINUE
C     Return to calling subroutine.
      RETURN
C     No extinctions.
   67 WRITE(LPT,68)
   68 FORMAT(/'Spacegroup has no extinction conditions')
C     Return to calling subroutine.
      RETURN
C     *******
      END
      SUBROUTINE HKLGEN
C     *******
C     Generate hkl listing.
C     Use standard variable types for reals and integers.
C     NB Characters are explicitly defined.
      IMPLICIT REAL*4 (A-H,O-Z)
      IMPLICIT INTEGER*4 (I-N)
C     Input variables  :
C     Output variables :
C     Subroutines used :
C     Functions called :
      PARAMETER (NDIM1=4000)
      CHARACTER*1 ICH,ISPG,ISYM,IDEF
      COMMON/HKL1/ICH(24),ISPG(20),ISYM(6,3),IDEF(6)
      COMMON/HKL2/IH(NDIM1),IK(NDIM1),IL(NDIM1),MUL(NDIM1),D(NDIM1),
     +  TH(NDIM1),IND(NDIM1),ICD(NDIM1)
      COMMON/HKL3/IOR,IOW,LPT,IOUT
      COMMON/HKL4/PI,RADS,RADS2,NBIT,NREF,NTOT,NCYC,MAXHKL
      COMMON/HKL5/A,B,C,AL,BE,GA,COSA,AR2,BR2,CR2,BCR,CAR,ABR
      COMMON/HKL6/LTP,LAU,NEX,IEX(32)
      COMMON/HKL7/DMIN,WAV2,ZER,EMAX
CMOD*
C     Extra variable for energy dispersive mode
C     A.D. Murray (11-01-88)
      COMMON /PDPL3/ DFCAOK,ENERGY
      LOGICAL        DFCAOK,ENERGY
C     Determine limits.
      DL = 1.0 / DMIN
      QMAX = DL * DL
      MAXH = INT(A * DL) + 1
      MAXK = INT(B * DL) + 1
      MAXL = INT(C * DL) + 1
      WRITE(LPT,1) MAXH,MAXK,MAXL
    1 FORMAT(/'Maximum indices are ',I3,' H ',I3,' K ',I3,' L ')
C     Loop through all reflections.
C     1 Triclinic l >= 0 exc. hk0, k < 0, and exc. h00 h <= 0
C     2 Monoclinic (a-unique) h >= 0, k >= 0 exc. h0l, l < 0
C     3            (b-unique) k >= 0, l >= 0 exc. hk0, h < 0
C     4            (c-unique) h >= 0, l >= 0 exc. 0kl, k < 0
C     5 Orthorhombic h >= 0, k >= 0, l >= 0
C     6 Tetragonal a) h >= 0, k >= 0, l >= 0 exc. 0kl
C     7            b) h >= k >= 0, l >= 0
C     8 Trigonal (Rhombohedral) a) h >= k >= 0 exc. 0kl, l < 0, and
C                                  exc. hkl, h <= l and k < l.
C     9                         b) h >= k >= 0 exc. 0kl, and
C                                  exc. hkl, k < l.
C     10         (Hexagonal) a) h > - k, k >= 0, l >= 0 exc. hk0, h < 0
C     11                     b) h >= 0, k >= 0, l >= 0 exc. hk0, h < k
C     12                     c) h >= 0, h >= - 2k, h >= k, l >= 0
C                               exc. hk0 k < 0
C     13 Hexagonal a) h >= 0, k >= 0, l >= 0 exc. 0kl
C     14           b) h >= k >= 0, l >= 0
C     15 Cubic a) h >= k >= 0, l >= 0 exc. hkl, h <= l and k < l
C     16       b) h >= k >= l >= 0
      N = NTOT
      MINL = 0
      IF (LAU.EQ.2.OR.LAU.EQ.8.OR.LAU.EQ.9) MINL = - MAXL
      DO 82 NL = MINL,MAXL
      GOTO (2,3,5,4,5,5,5,5,6) LAU
      GOTO (5,5,2,5,5,5,7) LAU - 9
    2 MINK = - MAXK
      IF (NL.EQ.0) MINK = 0
      GOTO 8
    3 MINK = 0
      IF (NL.LT.0) MINK = 1
      GOTO 8
    4 MINK = - MAXK
      GOTO 8
    5 MINK = 0
      GOTO 8
    6 MINK = MAX0(NL,0)
      GOTO 8
    7 MINK = NL
    8 DO 81 NK = MINK,MAXK
      GOTO (9,12,10,11,12,18,20,13,14) LAU
      GOTO (15,16,17,18,20,19,20) LAU - 9
    9 MINH = - MAXH
      IF (NK.EQ.0.AND.NL.EQ.0) MINH = 0
      GOTO 21
   10 MINH = - MAXH
      IF (NL.EQ.0) MINH = 0
      GOTO 21
   11 MINH = 0
      IF (NK.LT.0) MINH = 1
      GOTO 21
   12 MINH = 0
      GOTO 21
   13 MINH = NK
      IF (NL.LT.0) MINH = 1
      IF (NK.LT.NL) MINH = NL + 1
      GOTO 21
   14 MINH = MAX0(NK,1)
      IF (NK.EQ.0) MINH = - NL
      GOTO 21
   15 MINH = MIN0((1 - NK),0)
      IF (NL.EQ.0) MINH = 1
      GOTO 21
   16 MINH = 0
      IF (NL.EQ.0) MINH = NK
      GOTO 21
   17 MINH = MAX0(NK,- (2 * NK))
      GOTO 21
   18 MINH = MIN0(NK,1)
      GOTO 21
   19 MINH = NK
      IF (NK.LT.NL) MINH = NL + 1
      GOTO 21
   20 MINH = NK
   21 DO 80 NH = MINH,MAXH
      Q = NH * NH * AR2 + NK * NK * BR2 + NL * NL * CR2 +
     +    NK * NL * BCR + NL * NH * CAR + NH * NK * ABR
C     Reject 0,0,0 and h,k,l outside maximum 2-theta.
      IF (Q.EQ.0.0.OR.Q.GE.QMAX) GOTO 80
C     Check extinctions.
      IF (NEX.EQ.0) GOTO 55
C     The following extinctions may occur.
C     (1)   hkl : k + l = 2n.
C     (2)   hkl : h + l = 2n.
C     (3)   hkl : h + k = 2n.
C     (4)   hkl : h + k + l = 2n.
C     (5)   hkl : - h + k + l = 3n.
C     (6)   h00 : h = 2n.
C     (7)   0k0 : k = 2n.
C     (8)   00l : l = 2n.
C     (9)   hk0 : h = 2n.
C     (10)  0kl : k = 2n.
C     (11)  h0l : l = 2n.
C     (12)  h0l : h = 2n.
C     (13)  hk0 : k = 2n.
C     (14)  0kl : l = 2n.
C     (15)  hk0 : h + k = 2n.
C     (16)  h0l : h + l = 2n.
C     (17)  0kl : k + l = 2n.
C     (18)  hk0 : h + k = 4n.
C     (19)  h0l : h + l = 4n.
C     (20)  0kl : k + l = 4n.
C     (21)  00l : l = 4n.
C     (22)  hhl : l = 2n.
C     (23)  hhl : h = 2n.
C     (24)  hh0 : h = 2n.
C     (25)  hhl : h + l = 2n.
C     (26)  hhl : 2h + l = 4n.
C     (27)  00l : l = 3n.
C     (28)  00l : l = 6n.
C     (29)  hkk : h = 2n.
C     (30)  h00 : h = 4n.
C     (31)  2h-hl : l = 2n.
C     (32)  hkk : h + 2k = 4n.
      DO 54 I=1,NEX
      GOTO (22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,
     +42,43,44,45,46,47,48,49,50,51,52,53) IEX(I)
      CALL ETRAP
C     General extinctions.
C     k + l = 2n for hkl.
   22 IF (MOD((NK + NL),2).EQ.0) GOTO 54
      GOTO 80
C     l + h = 2n for hkl.
   23 IF (MOD((NL + NH),2).EQ.0) GOTO 54
      GOTO 80
C     h + k = 2n for hkl.
   24 IF (MOD((NH + NK),2).EQ.0) GOTO 54
      GOTO 80
C     h + k + l = 2n for hkl.
   25 IF (MOD((NH + NK + NL),2).EQ.0) GOTO 54
      GOTO 80
C     - h + k + l = 3n for hkl.
   26 IF (MOD((- NH + NK + NL),3).EQ.0) GOTO 54
      GOTO 80
C     Special extinctions.
C     h = 2n for h00.
   27 IF (NK.NE.0.OR.NL.NE.0) GOTO 54
      IF (MOD(NH,2).EQ.0) GOTO 54
      GOTO 80
C     k = 2n for 0k0.
   28 IF (NH.NE.0.OR.NL.NE.0) GOTO 54
      IF (MOD(NK,2).EQ.0) GOTO 54
      GOTO 80
C     l = 2n for 00l.
   29 IF (NH.NE.0.OR.NK.NE.0) GOTO 54
      IF (MOD(NL,2).EQ.0) GOTO 54
      GOTO 80
C     h = 2n for hk0.
   30 IF (NL.NE.0) GOTO 54
      IF (MOD(NH,2).EQ.0) GOTO 54
      GOTO 80
C     k = 2n for 0kl.
   31 IF (NH.NE.0) GOTO 54
      IF (MOD(NK,2).EQ.0) GOTO 54
      GOTO 80
C     l = 2n for h0l.
   32 IF (NK.NE.0) GOTO 54
      IF (MOD(NL,2).EQ.0) GOTO 54
      GOTO 80
C     h = 2n for h0l.
   33 IF (NK.NE.0) GOTO 54
      IF (MOD(NH,2).EQ.0) GOTO 54
      GOTO 80
C     k = 2n for hk0.
   34 IF (NL.NE.0) GOTO 54
      IF (MOD(NK,2).EQ.0) GOTO 54
      GOTO 80
C     l = 2n for 0kl.
   35 IF (NH.NE.0) GOTO 54
      IF (MOD(NL,2).EQ.0) GOTO 54
      GOTO 80
C     h + k = 2n for hk0.
   36 IF (NL.NE.0) GOTO 54
      IF (MOD((NH + NK),2).EQ.0) GOTO 54
      GOTO 80
C     h + l = 2n for h0l.
   37 IF (NK.NE.0) GOTO 54
      IF (MOD((NL + NH),2).EQ.0) GOTO 54
      GOTO 80
C     k + l = 2n for 0kl.
   38 IF (NH.NE.0) GOTO 54
      IF (MOD((NK + NL),2).EQ.0) GOTO 54
      GOTO 80
C     h + k = 4n for hk0.
   39 IF (NL.NE.0) GOTO 54
      IF (MOD((NH + NK),4).EQ.0) GOTO 54
      GOTO 80
C     h + l = 4n for h0l.
   40 IF (NK.NE.0) GOTO 54
      IF (MOD((NL + NH),4).EQ.0) GOTO 54
      GOTO 80
C     k + l = 4n for 0kl.
   41 IF (NH.NE.0) GOTO 54
      IF (MOD((NK + NL),4).EQ.0) GOTO 54
      GOTO 80
C     l = 4n for 00l.
   42 IF ((NH + NK).NE.0) GOTO 54
      IF (MOD(NL,4).EQ.0) GOTO 54
      GOTO 80
C     l = 2n for hhl.
   43 IF ((NH - NK).NE.0) GOTO 54
      IF (MOD(NL,2).EQ.0) GOTO 54
      GOTO 80
C     h = 2n for hhl.
   44 IF ((NH - NK).NE.0) GOTO 54
      IF (MOD(NH,2).EQ.0) GOTO 54
      GOTO 80
C     h = 2n for hh0.
   45 IF ((NH - NK).NE.0.OR.NL.NE.0) GOTO 54
      IF (MOD(NH,2).EQ.0) GOTO 54
      GOTO 80
C     h + l = 2n for hhl.
   46 IF ((NH - NK).NE.0) GOTO 54
      IF (MOD((NH + NL),2).EQ.0) GOTO 54
      GOTO 80
C     2h + l = 4n for hhl.
   47 IF ((NH - NK).NE.0) GOTO 54
      IF (MOD((NH + NH + NL),4).EQ.0) GOTO 54
      GOTO 80
C     l = 3n for 00l.
   48 IF (NH.NE.0.OR.NK.NE.0) GOTO 54
      IF (MOD(NL,3).EQ.0) GOTO 54
      GOTO 80
C     l = 6n for 00l.
   49 IF (NH.NE.0.OR.NK.NE.0) GOTO 54
      IF (MOD(NL,6).EQ.0) GOTO 54
      GOTO 80
C     h = 2n for hkk.
   50 IF (NK.NE.NL) GOTO 54
      IF (MOD(NH,2).EQ.0) GOTO 54
      GOTO 80
C     h = 4n for h00.
   51 IF (NK.NE.0.OR.NL.NE.0) GOTO 54
      IF (MOD(NH,4).EQ.0) GOTO 54
      GOTO 80
C     l = 2n for 2h-hl.
   52 IF (NH.NE.-(NK + NK)) GOTO 54
      IF (MOD(NL,2).EQ.0) GOTO 54
      GOTO 80
C     h + 2k = 4n for hkk.
   53 IF (NK.NE.NL) GOTO 54
      IF (MOD((NH + NK + NK),4).EQ.0) GOTO 54
      GOTO 80
   54 CONTINUE
   55 CONTINUE
C     Calculate multiplicity.
C     1 Triclinic J = 2 all h,k,l.
C     2 Monoclinic (a-unique) J = 2 for 0kl or h00, others J = 4.
C     3            (b-unique) J = 2 for h0l or 0k0, others J = 4.
C     4            (c-unique) J = 2 for hk0 or 00l, others J = 4.
C     5 Orthorhombic J = 2 for h00, 0k0, or 00l, J = 4 for hk0, h0l,
C                    or 0kl, others J = 8.
C     6 Tetragonal a) J = 2 for 00l, J = 4 for h00, 0k0, or hk0,
C                     others J = 8.
C     7            b) J = 2 for 00l, J = 4 for h00 or hh0,
C                     J = 8 for h0l, hk0, or hhl, others J = 16.
C     8 Trigonal (Rhomohedral) a) J = 2 for hhh, others J = 6.
C     9                        b) J = 2 for hhh, J = 6 for hhl, hkk,
C                                 or h0-l, others J = 12.
C     10         (Hexagonal) a) J = 2 for 00l, others J = 6.
C     11                     b) J = 2 for 00l, J = 6 for h0l, 0kl,
C                               or hh0, others J = 12.
C     12                     c) J = 2 for 00l, J = 6 for h00, hhl,
C                               or 2h-hl, others J = 12.
C     13 Hexagonal a) J = 2 for 00l, J = 6 for h00, hh0, or hk0,
C                     others J = 12
C     14           b) J = 2 for 00l, J = 6 for h00 or hh0, J = 12 for
C                     hk0, h0l, or hhl, others J = 24
C     15 Cubic a) J = 6 for h00, J = 8 for hhh, J = 12 for hh0 or hk0,
C                 others J = 24
C     16       b) J = 6 for h00, J = 8 for hhh, J = 12 for hh0,
C                 J = 24 for hk0, hhl, or hkk, others J = 48
      GOTO (71,56,57,58,59,60,61,62,63) LAU
      GOTO (64,65,66,67,68,69,70) LAU - 9
   56 IF (NH.EQ.0) GOTO 71
      IF (NK.EQ.0.AND.NL.EQ.0) GOTO 71
      GOTO 72
   57 IF (NK.EQ.0) GOTO 71
      IF (NL.EQ.0.AND.NH.EQ.0) GOTO 71
      GOTO 72
   58 IF (NL.EQ.0) GOTO 71
      IF (NH.EQ.0.AND.NK.EQ.0) GOTO 71
      GOTO 72
   59 IF ((NH + NK).EQ.0) GOTO 71
      IF ((NK + NL).EQ.0) GOTO 71
      IF ((NL + NH).EQ.0) GOTO 71
      IF (NH.EQ.0) GOTO 72
      IF (NK.EQ.0) GOTO 72
      IF (NL.EQ.0) GOTO 72
      GOTO 74
   60 IF ((NH + NK).EQ.0) GOTO 71
      IF (NL.EQ.0) GOTO 72
      GOTO 74
   61 IF ((NH + NK).EQ.0) GOTO 71
      IF (NL.EQ.0.AND.(NH.EQ.NK.OR.NK.EQ.0)) GOTO 72
      IF (NL.EQ.0.OR.NK.EQ.0.OR.NH.EQ.NK) GOTO 74
      GOTO 76
   62 IF (NH.EQ.NK.AND.NK.EQ.NL) GOTO 71
      GOTO 73
   63 IF (NH.EQ.NK.AND.NK.EQ.NL) GOTO 71
      IF (NH.EQ.NK.OR.NK.EQ.NL.OR.((NH + NL).EQ.0.AND.NK.EQ.0)) GOTO 73
      GOTO 75
   64 IF ((NH + NK).EQ.0) GOTO 71
      GOTO 73
   65 IF ((NH + NK).EQ.0) GOTO 71
      IF (NH.EQ.0.OR.NK.EQ.0.OR.(NL.EQ.0.AND.NH.EQ.NK)) GOTO 73
      GOTO 75
   66 IF ((NH + NK).EQ.0) GOTO 71
      IF (NH.EQ.NK.OR.NK.EQ.- (NH + NK).OR.(NL.EQ.0.AND.NK.EQ.0))
     +  GOTO 73
      GOTO 75
   67 IF ((NH + NK).EQ.0) GOTO 71
      IF (NL.EQ.0) GOTO 73
      GOTO 75
   68 IF ((NH + NK).EQ.0) GOTO 71
      IF ((NK + NL).EQ.0.OR.(NH.EQ.NK.AND.NL.EQ.0)) GOTO 73
      IF (NK.EQ.0.OR.NL.EQ.0.OR.NH.EQ.NK) GOTO 75
      GOTO 77
   69 IF ((NK + NL).EQ.0) GOTO 73
      IF (NH.EQ.NK.AND.NK.EQ.NL) GOTO 74
      IF (NK.EQ.0.OR.NL.EQ.0) GOTO 75
      GOTO 77
   70 IF ((NK + NL).EQ.0) GOTO 73
      IF (NH.EQ.NK.AND.NK.EQ.NL) GOTO 74
      IF (NH.EQ.NK.AND.NL.EQ.0) GOTO 75
      IF (NH.EQ.NK.OR.NK.EQ.NL.OR.NL.EQ.0) GOTO 77
      GOTO 78
C     Set multiplicity factor to 2, 4, 6, 8, 12, 16, 24, or 48.
   71 MF = 2
      GOTO 79
   72 MF = 4
      GOTO 79
   73 MF = 6
      GOTO 79
   74 MF = 8
      GOTO 79
   75 MF = 12
      GOTO 79
   76 MF = 16
      GOTO 79
   77 MF = 24
      GOTO 79
   78 MF = 48
C     Store h,k,l, and J. Calculate d-spacing and 2-theta.
C     Increment reflection counter.
   79 N = N + 1
C     Check next N does not exceed array limits !
      IF (N.GT.MAXHKL) GOTO 85
      IH(N) = NH
      IK(N) = NK
      IL(N) = NL
      MUL(N) = MF
C     Calculate d-spacing.
      D(N) = 1.0 / SQRT(Q)
CMOD*
C     Allow for energy dispersive diffraction with IF block.
C     A.D. Murray (11-01-88)
      IF (ENERGY) THEN
C     Calculate energy for Energy dispersive X-Ray scattering
      TH(N) = WAV2 * EMAX / D(N)
      ELSE
C     Calculate 2-theta (conventional diffraction).
C     Obs(2-theta) = Calc(2-theta) + Zero-point error.
      TH(N) = ASIN(WAV2 / D(N)) / RADS2 + ZER
      ENDIF
CMOD*
C     End of modified section
   80 CONTINUE
   81 CONTINUE
   82 CONTINUE
C     How many generated ?
      NREF = N - NTOT
      WRITE(IOW,83) NREF
   83 FORMAT(/1X,'Total number of reflections is ',I5)
      WRITE(LPT,84) NREF
   84 FORMAT(/'Total number of reflections is ',I5)
C     Return to calling subroutine.
      RETURN
C     Give error message if N exceeds array size.
   85 WRITE(IOW,86) MAXHKL
   86 FORMAT(/1X,'Warning - core limit reached : array size set at ',I5)
      WRITE(IOW,87) NH,NK,NL
   87 FORMAT(/1X,'Generation of HKL''s stopped at',3I4)
C     Abort.
      CALL ABORT
C     *******
      END
      SUBROUTINE LPTREF
C     *******
C     Use standard variable types for reals and integers.
C     NB Characters are explicitly defined.
      IMPLICIT REAL*4 (A-H,O-Z)
      IMPLICIT INTEGER*4 (I-N)
C     Input variables  :
C     Output variables :
C     Subroutines used :
C     Functions called :
      PARAMETER (NDIM1=4000)
      CHARACTER*1 ANS
      COMMON/HKL2/IH(NDIM1),IK(NDIM1),IL(NDIM1),MUL(NDIM1),D(NDIM1),
     +  TH(NDIM1),IND(NDIM1),ICD(NDIM1)
      COMMON/HKL3/IOR,IOW,LPT,IOUT
      COMMON/HKL4/PI,RADS,RADS2,NBIT,NREF,NTOT,NCYC,MAXHKL
      COMMON/HKL6/LTP,LAU,NEX,IEX(32)
CMOD*
C     Extra variable for energy dispersive mode
C     NOTE: In this subroutine several FORMAT statements have been
C     changed. The field '2-theta' has been removed so that the
C     character variable TH2ORE is output instead. TH2ORE is set
C     to either '2-theta' or 'E (KeV)' depending on the mode of
C     calculation.
C     A.D. Murray (11-01-88)
      COMMON /PDPL3/ DFCAOK,ENERGY
      LOGICAL        DFCAOK,ENERGY
      CHARACTER*7    TH2ORE
CMOD*
C     Extra variable for PDPL routines
      CHARACTER*80 LINEIN
CMOD*
C     Set TH2ORE - A.D. Murray (11-01-88)
          IF (ENERGY) THEN
          TH2ORE = 'E (KeV)'
          ELSE
          TH2ORE = '2-theta'
          ENDIF
CMOD* 
C     End of modified section.
      N = NTOT + NREF
C     Calculate number of reflections within the sphere of reflection.
      ISPHE = 0
      DO 1 I=(NTOT+1),N
    1 ISPHE = ISPHE + MUL(I)
      WRITE(LPT,2) ISPHE
    2 FORMAT(/'Sum of the multiplicities is ',I6)
C     Keep tabs on each reflection.
      DO 3 I=(NTOT+1),N
    3 IND(I) = I
C     In practice reflections are always required sorted. However
C     during testing of this type of program, it is often useful
C     to be able to list the reflections in the order generated.
      IF (NREF.EQ.1) GOTO 8
CMOD*
C     Extra statement label
C     A.D. Murray (06-01-88)
 312  CONTINUE
      WRITE(IOW,4)
CMOD*
C     Use PDPL routine PROMPT instead of VAX $ 
C     A.D. Murray (07-01-88)
C#  4 FORMAT(/1X,'Sort reflections ? (<CR> = yes)'/1X,'->  ',$)
    4 FORMAT(/1X,'Sort reflections ? (<CR> = yes)')
      CALL PROMPT('Sort ? > ',9,IOW)
CMOD*
C     Use PDPL routine COMINT to get input
C     A.D. Murray (06-01-88)
C#    READ(IOR,5,END=32) ANS
      CALL COMINT(0,LINEIN,IFLAG,'SORTIN')
      IF (IFLAG.EQ.2) GOTO 32
      IF (IFLAG.EQ.3) GOTO 312
      ANS = LINEIN(1:1)
CMOD*
C     End of modified section
    5 FORMAT(A1)
      IF (ANS.NE.' '.AND.ANS.NE.'Y'.AND.ANS.NE.'y') GOTO 8
      M = NTOT + 1
C     Sort from M to N.
      CALL SORTER(M,N)
C     Calculate number of unique reflections, i.e. those with
C     non-identical d-spacings. (Easily done when reflections have
C     been sorted).
      NUN = 1
      DO 6 I=(NTOT+2),N
C     Is d-spacing same as previous reflection (within computer error).
      IF (ABS(D(IND(I)) - D(IND(I-1))).LT.0.00001) GOTO 6
      NUN = NUN + 1
    6 CONTINUE
C     Write to LPT file.
      WRITE(LPT,7) NUN
    7 FORMAT(/'Number of unique reflections is ',I5)
C     Skip after first pattern.
    8 CONTINUE
C     TTY output.
      WRITE(IOW,9) TH2ORE
C#  9 FORMAT(/4X,'H',3X,'K',3X,'L',4X,'J',8X,'D',5X,'2-theta'/)
    9 FORMAT(/4X,'H',3X,'K',3X,'L',4X,'J',8X,'D',5X,A7/)
      DO 10 I=(NTOT+1),MIN0((NTOT+10),N)
   10 WRITE(IOW,11) IH(IND(I)),IK(IND(I)),IL(IND(I)),MUL(IND(I)),
     + D(IND(I)),TH(IND(I))
   11 FORMAT(1X,3I4,I5,1X,F10.4,1X,F8.2)
C     Write out list to LPT.
C     Special for rhombohedral systems. (Both sets of h,k,l given).
   12 IF (LTP.EQ.7) GOTO 24
C     For hexagonal axes, give h,k,i,l.
      IF (LAU.GE.10.AND.LAU.LE.14) GOTO 20
C     For cubic axes, also list h*h+k*k+l*l.
      IF (LAU.EQ.15.OR.LAU.EQ.16) GOTO 16
C     LPT output - general.
      WRITE(LPT,13) TH2ORE
C# 13 FORMAT(/3X,'H',3X,'K',3X,'L',4X,'J',8X,'D',5X,'2-theta'/)
   13 FORMAT(/3X,'H',3X,'K',3X,'L',4X,'J',8X,'D',5X,A7/)
      DO 14 I=(NTOT+1),N
   14 WRITE(LPT,15) IH(IND(I)),IK(IND(I)),IL(IND(I)),MUL(IND(I)),
     +D(IND(I)),TH(IND(I))
   15 FORMAT(3I4,I5,1X,F10.4,1X,F8.2)
C     Return to calling subroutine.
      RETURN
C     LPT output - cubic.
   16 WRITE(LPT,17) TH2ORE
C# 17 FORMAT(/3X,'H',3X,'K',3X,'L',4X,'J',8X,'D',5X,'2-theta',
   17 FORMAT(/3X,'H',3X,'K',3X,'L',4X,'J',8X,'D',5X,A7,
     +  5X,'H*H + K*K + L*L')
      DO 18 I=(NTOT+1),N
      NSUM = IH(IND(I)) * IH(IND(I)) + IK(IND(I)) * IK(IND(I)) +
     +  IL(IND(I)) * IL(IND(I))
   18 WRITE(LPT,19) IH(IND(I)),IK(IND(I)),IL(IND(I)),MUL(IND(I)),
     +D(IND(I)),TH(IND(I)),NSUM
   19 FORMAT(3I4,I5,1X,F10.4,1X,F8.2,9X,I5)
C     Return to calling subroutine.
      RETURN
C     LPT output - hexagonal.
   20 WRITE(LPT,21) TH2ORE
C# 21 FORMAT(/3X,'H',3X,'K',3X,'I',3X,'L',4X,'J',8X,'D',5X,'2-theta'/)
   21 FORMAT(/3X,'H',3X,'K',3X,'I',3X,'L',4X,'J',8X,'D',5X,A7/)
      DO 22 I=(NTOT+1),N
C     The extra index, i, is given by : i = - h - k.
      II = - IH(IND(I)) - IK(IND(I))
   22 WRITE(LPT,23) IH(IND(I)),IK(IND(I)),II,IL(IND(I)),MUL(IND(I)),
     + D(IND(I)),TH(IND(I))
   23 FORMAT(4I4,I5,1X,F10.4,1X,F8.2)
C     Return to calling subroutine.
      RETURN
C     LPT output - rhombohedral systems.
   24 IF (LAU.EQ.8.OR.LAU.EQ.9) GOTO 28
C     Hexagonal generated.
      WRITE(LPT,25) TH2ORE
C# 25 FORMAT(/3X,'H',3X,'K',3X,'I',3X,'L',4X,'J',8X,'D',5X,'2-theta',
   25 FORMAT(/3X,'H',3X,'K',3X,'I',3X,'L',4X,'J',8X,'D',5X,A7,
     +  3X,'(Rhombohedral indices h,k,l)'/)
      DO 26 I=(NTOT+1),N
C     The extra index, i, is given by : i = - h - k.
      II = - IH(IND(I)) - IK(IND(I))
C     h(R) = [2h(H) + k(H) + l(H)] / 3;
C     k(R) = [- h(H) + k(H) + l(H)] / 3;
C     l(R) = [- h(H) - 2k(H) + l(H)] / 3.
      MH = (2 * IH(IND(I)) + IK(IND(I)) + IL(IND(I))) / 3
      MK = (- IH(IND(I)) + IK(IND(I)) + IL(IND(I))) / 3
      ML = (- IH(IND(I)) - 2 * IK(IND(I)) + IL(IND(I))) / 3
   26 WRITE(LPT,27) IH(IND(I)),IK(IND(I)),II,IL(IND(I)),MUL(IND(I)),
     +D(IND(I)),TH(IND(I)),MH,MK,ML
   27 FORMAT(4I4,I5,1X,F10.4,1X,F8.2,11X,3I4)
C     Return to calling subroutine.
      RETURN
C     Rhombohedral generated.
   28 WRITE(LPT,29) TH2ORE
C# 29 FORMAT(/3X,'H',3X,'K',3X,'L',4X,'J',8X,'D',5X,'2-theta',
   29 FORMAT(/3X,'H',3X,'K',3X,'L',4X,'J',8X,'D',5X,A7,
     +  3X,'(Hexagonal indices h,k,i,l)'/)
      DO 30 I=(NTOT+1),N
C     h(H) = h(R) - k(R); k(H) = k(R) - l(R); i(H) = l(R) - h(R);
C     l(H) = h(R) + k(R) + l(R).
      MH = IH(IND(I)) - IK(IND(I))
      MK = IK(IND(I)) - IL(IND(I))
C     The extra index, i, is given by : i = - h - k.
      MI = - MH - MK
      ML = IH(IND(I)) + IK(IND(I)) + IL(IND(I))
   30 WRITE(LPT,31) IH(IND(I)),IK(IND(I)),IL(IND(I)),MUL(IND(I)),
     +D(IND(I)),TH(IND(I)),MH,MK,MI,ML
   31 FORMAT(3I4,I5,1X,F10.4,1X,F8.2,8X,4I4)
C     Return to calling subroutine.
      RETURN
C     EOF on input, abort to monitor !
   32 CALL EOFINP
C     *******
      END
      SUBROUTINE SORTER(M,N)
C     *******
C     Use standard variable types for reals and integers.
C     NB Characters are explicitly defined.
      IMPLICIT REAL*4 (A-H,O-Z)
      IMPLICIT INTEGER*4 (I-N)
C     Input variables  : M      N      TH     IND
C     Output variables :
C     Subroutines used :
C     Functions called :
      PARAMETER (NDIM1=4000)
      COMMON/HKL2/IH(NDIM1),IK(NDIM1),IL(NDIM1),MUL(NDIM1),D(NDIM1),
     +  TH(NDIM1),IND(NDIM1),ICD(NDIM1)
C     Sort HKL values to go from low to high 2-theta, i.e. large
C     to small d-spacing or long to short T.O.F.
C     N.B. It is necessary to sort using 2-theta values since for
C     multipattern refinements using 2 or more wavelengths
C     (e.g. Cu K-alpha1 & Cu K-alpha2), the reflections are not
C     in order of d-spacing. For each single pattern, d-spacing
C     or 2-theta angle could be used.
      DO 2 J=M,(N-1)
      NOSWOP = 1
      DO 1 I=M,(M-1+N-J)
C     Test and skip if in correct order.
      IF (TH(IND(I)).LT.TH(IND(I+1))) GOTO 1
C     Otherwise swop index array.
      NOSWOP = 0
      ITMP = IND(I)
      IND(I) = IND(I+1)
      IND(I+1) = ITMP
    1 CONTINUE
C     If noswop, skip out of loop.
      IF (NOSWOP.EQ.1) RETURN
    2 CONTINUE
C     Return to calling subroutine.
      RETURN
C     *******
      END
      SUBROUTINE FILOUT(IFORM)
C     *******
C     Use standard variable types for reals and integers.
C     NB Characters are explicitly defined.
      IMPLICIT REAL*4 (A-H,O-Z)
      IMPLICIT INTEGER*4 (I-N)
C     Input variables  : IFORM
C     Output variables :
C     Subroutines used :
C     Functions called :
      PARAMETER (NDIM1=4000)
      COMMON/HKL2/IH(NDIM1),IK(NDIM1),IL(NDIM1),MUL(NDIM1),D(NDIM1),
     +  TH(NDIM1),IND(NDIM1),ICD(NDIM1)
      COMMON/HKL3/IOR,IOW,LPT,IOUT
      COMMON/HKL4/PI,RADS,RADS2,NBIT,NREF,NTOT,NCYC,MAXHKL
CMOD*
C     Variables required for PDPL routine GFILE (see later)
C     A.D. Murray (04-01-88)
      CHARACTER*30 DUMMY
      LOGICAL      ERROR
CMOD*
C     End of extra variable list

C     3 output formats - all very similar.
CMOD* 
C     Added 4th format - H,K,L and angle
C     A.D. Murray (07-01-88)
C#    GOTO (1,5,11) IFORM
      GOTO (1,5,11,131) IFORM
C     Hewat PROF1 output.
    1 WRITE(IOUT,2) NTOT
    2 FORMAT(I8/3(7X,'1'))
      DO 3 I=1,NTOT
    3 WRITE(IOUT,4) ICD(IND(I)),IH(IND(I)),IK(IND(I)),IL(IND(I)),
     + MUL(IND(I))
    4 FORMAT(5I8,4X,'0000')
      GOTO 14
C     Bendall MPREP output.
C     Write out number of reflections to file.
    5 WRITE(IOUT,6) NTOT
    6 FORMAT(I8)
C     Write out coefficients of Miller indices to file !
      DO 7 I=1,NCYC
    7 WRITE(IOUT,8)
    8 FORMAT(3(7X,'1'))
      DO 9 I=1,NTOT
    9 WRITE(IOUT,10) ICD(IND(I)),IH(IND(I)),IK(IND(I)),IL(IND(I)),
     + MUL(IND(I))
   10 FORMAT(5I8)
      GOTO 14
C     Cockcroft PREPRO output.
   11 WRITE(IOUT,6) NTOT
      DO 12 I=1,NTOT
      NCD = ICD(IND(I))
      NPAT = NCD / 10
      NTYP = NCD - NPAT * 10
   12 WRITE(IOUT,13) IH(IND(I)),IK(IND(I)),IL(IND(I)),
     + MUL(IND(I)),NPAT,NTYP
   13 FORMAT(4I8,4X,2I2)
CMOD*
C     Write out H,K,L and 2Theta
C     A.D. Murray (07-01-88)
      GOTO 14
  131 CONTINUE

        DO 132 I = 1,NTOT
        INDX = IND(I)
        WRITE(IOUT,133) IH(INDX),IK(INDX),IL(INDX),TH(INDX)
  132   CONTINUE

  133 FORMAT(3I8,4X,F8.3)
CMOD*
C     End of modified section
C     Close output file.
CMOD*
C     Use PDPL routine GFILE for file CLOSEing
C     A.D. Murray (04-01-88)
C# 14 CLOSE(UNIT=IOUT)
   14 CALL GFILE(IOUT,DUMMY,'CLS','FOR',0,ERROR,IOW)
C     Return to calling subroutine.
      RETURN
C     *******
      END
      SUBROUTINE ABORT
C     *******
C     Use standard variable types for reals and integers.
      IMPLICIT INTEGER*4 (I-N)
C     Input variables  : LPT    IOUT
C     Output variables :
C     Subroutines used : EXIT
C     Functions called :
      COMMON/HKL3/IOR,IOW,LPT,IOUT
      COMMON /PDPL1/ LPTFIL,AFILE,HLPFIL
      CHARACTER*30   LPTFIL,AFILE,HLPFIL
C     Delete output file(s).
CMOD*
C     Use PDPL routine GFILE for file CLOSEing
C     A.D. Murray (04-01-88)
C#    CLOSE(UNIT=LPT,DISPOSE='DELETE')
C#    IF (IOUT.NE.0) CLOSE(UNIT=IOUT,DISPOSE='DELETE')
      CALL GFILE(LPT,LPTFIL,'CLD','FOR',0,ERROR,IOW)
      IF (IOUT.NE.0) CALL GFILE(IOUT,AFILE,'CLD','FOR',ERROR,IOW)
C     Exit program.
C     Use PDPL routine ENDALL for ending program
C     A.D. Murray (04-01-88)
C#    CALL ENDCPU
      CALL ENDALL('DRAGON    ')
C     *******
      END
      SUBROUTINE ETRAP
C     *******
C     Give error message if error in program logic detected -
C     hopefully this should never occur !
C     Use standard variable types for reals and integers.
      IMPLICIT INTEGER*4 (I-N)
C     Input variables  : IOW    LPT
C     Output variables :
C     Subroutines used : EXIT
C     Functions called :
      COMMON/HKL3/IOR,IOW,LPT,IOUT
      WRITE(IOW,1)
    1 FORMAT(/1X,'Internal program fault')
C     Abort.
      CALL ABORT
C     *******
      END
      SUBROUTINE EOFINP
C     *******
C     EOF on input, abort to monitor !
C     Use standard variable types for reals and integers.
      IMPLICIT INTEGER*4 (I-N)
C     Input variables  : IOW    LPT
C     Output variables :
C     Subroutines used : EXIT
C     Functions called :
      COMMON/HKL3/IOR,IOW,LPT,IOUT
      WRITE(IOW,1)
    1 FORMAT(/1X,'End of file encountered in input')
C     Abort.
      CALL ABORT
C     *******
      END

      SUBROUTINE GETEDS
C     *******
C     Reads in detector angle & maximum energy for energy dispersive
C     calculations. Set wavelength equal to the correct value for
C     calculation of d-spaceing range
C     Based on DRAGON routine GETWAV
C     A.D. Murray (11-01-88)
C     Use standard variable types for reals and integers.
C     NB Characters are explicitly defined.
      IMPLICIT REAL*4 (A-H,O-Z)
      IMPLICIT INTEGER*4 (I-N)
C     Input variables  : IOW    DFDANG DFEMAX IOR    LPT    RADS
C     Output variables : WAV    TMX    ZER    WAV2   DMIN
C     Subroutines used : EOFINP
C     Functions called :
      PARAMETER (NDIM1=4000,NDIM2=75)
      CHARACTER*1 TEXT
      COMMON/HKL3/IOR,IOW,LPT,IOUT
      COMMON/HKL4/PI,RADS,RADS2,NBIT,NREF,NTOT,NCYC,MAXHKL
      COMMON/HKL7/DMIN,WAV2,ZER,EMAX
      COMMON/HKL9/TEXT(NDIM2)
      COMMON /PDPL2/ DFCELL(6),DFWAV,DZP
      COMMON /PDPL3/ DFCAOK,ENERGY
      LOGICAL        DFCAOK,ENERGY
      CHARACTER*80 LINEIN
      DOUBLE PRECISION VAL
      LOGICAL ERROR
      DIMENSION VAL(3)
      DATA NVAL/2/
C     Set default detector angle to 10 degrees
      DATA DFDANG/10.0000/
C     Set default maximum energy
      DATA DFEMAX/100.0/
C     Get detector angle and maximum energy limit.
 309  CONTINUE
      WRITE(IOW,1) DFDANG,DFEMAX
    1 FORMAT(/1X,'Give detector angle (degrees) and maximum energy',
     +  1X,'(KeV)'
     +  /1X,'(<CR> = ',F7.3,' degrees, ',F8.3,' KeV)')
      CALL PROMPT('Detector angle > ',17,IOW)
      CALL COMINT(0,LINEIN,IFLAG,'WAVELE')
      IF (IFLAG.EQ.2) GOTO 7
      IF (IFLAG.EQ.3) GOTO 309
      CALL DIGITS(LINEIN,VAL,2,1,KEND,NFND,ERROR,IOW)
      IF (ERROR) GOTO 309
      DANG = VAL(1)
      EMAX = VAL(2)
C     Check detector angle and max. energy
      IF (DANG.LT.0.0) GOTO 5
      IF (EMAX.LT.0.0) GOTO 5
C     If <CR>, then set defaults.
      IF (DANG.EQ.0.0) DANG = DFDANG
C     Set default maximum energy
      IF (EMAX.EQ.0.0) EMAX = DFEMAX
C     Store defaults.
      DFDANG = DANG
      DFEMAX = EMAX
      WRITE(LPT,3) DANG,EMAX
    3 FORMAT(/'Energy dispersive mode - Detector angle ',F8.3,
     +  1X,'deg   Max. energy ',F8.3,' KeV')
C     Calculate minimum d-spacing.
C     Set wavelength and max. two-theta to allow calculation of
C     correct range of d-spacings
      WAV = 12.39811616D0 / (EMAX * SIN(DANG * RADS2))
      TMX = 180.0
      ZER = 0.0
      WAV2 = WAV / 2.0
C     Take into account zero-point - only relevant when very large !
      DMIN = WAV2 / SIN((TMX - ZER) * RADS2)
      WRITE(LPT,4) DMIN
    4 FORMAT(/'Minimum d-spacing is ',F8.5,' A')
C     Return to calling subroutine.
      RETURN
C     Give error message if error in wavelength.
    5 WRITE(IOW,6)
    6 FORMAT(/1X,'Impossible detector angle, max. energy, etc.')
C     Allow another attempt if mistakes are made !!
      GOTO 309
C     EOF on input, abort to monitor !
    7 CALL EOFINP
C     *******
      END
      BLOCK DATA
CMOD*
C     Block data module added by A.D. Murray (04-01-88) because
C     DATA statement initialisation of variables in COMMON blocks
C     is only allowed in BLOCK DATA statements in Standard Fortran 77.

      COMMON/HKL1/ICH(24),ISPG(20),ISYM(6,3),IDEF(6)
      CHARACTER*1 ICH,ISPG,ISYM,IDEF

      DATA ICH/'P','A','B','C','F','I','R','1','2','3','4','5','6',
     +  'm','a','b','c','n','d','-','/','(',')',' '/
      END
